// Dutchtronix AVR Oscilloscope Clock
//
//  Copyright  2010 Johannes P.M. de Rie
//
//  All Rights Reserved
//
//  This file is part of the Dutchtronix Oscilloscope Clock Distribution.
//  Use, modification, and re-distribution is permitted subject to the
//  terms in the file named "LICENSE.TXT", which contains the full text
//  of the legal notices and should always accompany this Distribution.
//
//  This software is provided "AS IS" with NO WARRANTY OF ANY KIND.
//
//  This notice (including the copyright and warranty disclaimer)
//  must be included in all copies or derivations of this software.
//
// Firmware for Dutchtronix AVR Oscilloscope Clock
//
#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

#include "./config.h"
#include "../../winavr/ClkDebug.h"

extern char MsgShortSecond[] PROGMEM;
extern char MsgPszShortSecCnt[] PROGMEM;
extern char MsgPszLongSecCnt[] PROGMEM;
extern char MsgPszAssert[] PROGMEM;

extern void SpecialUARTSB(char ch);
extern void UARTPrintfProgStr(PGM_P pBuf);
extern void UsrPrintNewLine(void);

void DebugSerialOut(char ch);
void DebugSerialOutStr(PGM_P pSrc);
void DebugSerialu08(byte b);
void DebugSerialu16(uint w);

#if DEBUG
void Special_UARTPfu08(byte b);
void Special_UARTPfu16(uint w);

void DebugAssert(uint line)
{
	DebugSerialOutStr(MsgPszAssert);
	DebugSerialu16(line);
	DebugSerialOutStr(MsgPszNewLine);
	cli();
	do {} while (TRUE);
}
#if SCOPECLOCK

void ShowSecSkippedMsg(void){
	char ch, *p = MsgShortSecond;
	while ((ch = pgm_read_byte(p++)) != 0) {
		SpecialUARTSB(ch);
	} 
}

void ShowRefreshCnt(void)
{
	Special_UARTPfu16(RefreshCnt);
	SpecialUARTSB(' ');
}

#if NETFREQUENCY
void ShowNetCycleCnt(void)
{
	Special_UARTPfu08(NetCycleCnt);
	SpecialUARTSB(' ');
}
#endif

//
// print byte value as 2 hex chars
//
void Special_UARTPfu08(byte b)
{
	SpecialUARTSB(Hex2AscNew(b >> 4));
	SpecialUARTSB(Hex2AscNew(b & 0x0f));
}
//
// print word value as 4 hex chars
//
void Special_UARTPfu16(uint w)
{
	Special_UARTPfu08(w >> 8);
	Special_UARTPfu08(w & 0xff);
}
//
// print long value as 8 hex chars
//

void UART_Printfu32(unsigned long l)
{
	Special_UARTPfu16(l >> 16);
	Special_UARTPfu16(l & 0xffff);
}
//
// Report Internal Variables
//
BOOL UsrDebugReport(void)
{
//UsrDebugReportNew:
	UARTPrintfProgStr(MsgPszShortSecCnt);
	Special_UARTPfu16(ShortSecCnt);
	UsrPrintNewLine();
	UARTPrintfProgStr(MsgPszLongSecCnt);
	Special_UARTPfu16(LongSecCnt);
	UsrPrintNewLine();
	return TRUE;
}
//
// UNIMPLEMENTED
//
BOOL UsrDebugSetting(byte mode)
{
	if (mode == 1) {
		ExtDbg1 ^= 1;
	} else if (mode == 2) {
		ExtDbg2 ^= 1;
	} else if (mode == 3) {
		ExtDbg1 ^= 1;
		ExtDbg2 ^= 1;
	}
	return TRUE;
}
#endif

#if 0		//UNUSED
//
// run a Time Set Test
//
void Usr_Test(void)
{
	byte cnt = 255;
	do {
		DBGStage = 5;
		do {
			ExtraTimer = 100;
			TestRun();
		} while (--DBGState);
	} while (--cnt);
}

void TestRun(void)
{
	byte cnt = 0;
	do {
		MainForever();
	} while (--cnt);
}

//
// callback function from TC1 interrupt.
//
void DBGTestFunc(void)
{
	if (DBGStage != 0) {
		if (DBGStage == 1) {
			GPIOR0 |= _BV(fUP);
		} else 
			GPIOR0 |= _BV(fTC0);
		}
		--DBGState;
		ExtraTimer = 100;
	}
}
#endif

#else	//!DEBUG

BOOL UsrDebugReport(void)
{
	return TRUE;
}

BOOL UsrDebugSetting(void)
{
	return TRUE;
}
#endif

#if DEBUG | ds1307
//
// DebugSerialOut(char ch)
//
void DebugSerialOut(char ch)
{
	while ((UCSR0A & _BV(UDRE0)) == 0) {}
	UDR0 = ch;
#if 0	//optional
	while ((UCSR0A & _BV(TXC0)) == 0) {}
#endif
}
//
// DebugSerialOutStr: Use when interrupts not enabled.
//
// IN: flash based string to send to serial port
//
void DebugSerialOutStr(PGM_P pSrc)
{
	char ch;
	while ((ch = pgm_read_byte(pSrc++)) != 0) {
		DebugSerialOut(ch);
	}
}
#endif

#if DEBUG
//
// print byte value as 2 hex chars. Works with interrupts off.
//
void DebugSerialu08(byte b)
{
	DebugSerialOut(Hex2AscNew(b >> 4));
	DebugSerialOut(Hex2AscNew(b & 0x0f));
}

//
// print word value as 4 hex chars. Works with interrupts off.
//
void DebugSerialu16(uint w)
{
	DebugSerialu08(w >> 8);
	DebugSerialu08(w & 0xff);
}
#endif
