// ClkISR.h -- Dutchtronix ISR Support
//
//  Copyright  2010 Johannes P.M. de Rie
//
//  All Rights Reserved
//
//  This file is part of the Dutchtronix Oscilloscope Clock Distribution.
//  Use, modification, and re-distribution is permitted subject to the
//  terms in the file named "LICENSE.TXT", which contains the full text
//  of the legal notices and should always accompany this Distribution.
//
//  This software is provided "AS IS" with NO WARRANTY OF ANY KIND.
//
//  This notice (including the copyright and warranty disclaimer)
//  must be included in all copies or derivations of this software.
//

#define	Timer0CountUp	101		// timer0 set for 6.464 mSec at 16 Mhz,
#define	TIMER0TIMEOUT  	51		// about 1/3 second
#define AUTOREPEATVAL	15
#define	Timer3CountUp 	1042	// timer1 set for 16.672 mSecs (60 Hz)
#define	TIMER3TIMEOUT  	1800	// about 30 seconds
#define ReloadSecondsCnt 60


typedef struct PushButtonState {
	byte	TC0Cntr;
	byte	Position;
	byte	KeyState;
	BOOL	fAutorepeat;
	byte	autorepeatcnt;
	BOOL	okToRepeat;
} PushButtonState_t;

extern PushButtonState_t S1;
extern PushButtonState_t S2;
extern PushButtonState_t S3;
extern PushButtonState_t S4;
extern uint	Timer3Counter;

#if DEBUG
byte	TC0DebugCnt;
#endif


extern byte SecondsCnt;
extern unsigned long TickCnt;

void InitTC0(void);
void InitTC3(void);
void InitSwitch(void);
void InitLed(void);
void BlinkLed0(void);
void BlinkLed1(void);

void WDTimerOff(void);
void InitiateSysReset(void);



extern volatile BOOL UARTOutReady;

extern void UARTKillPendingOutput(void);
extern void UARTClrRXBuffer(void);
extern BOOL UARTReceiveLine(char *pDst, byte maxLength);
extern void UARTFlushRecord(void);
extern void UARTSendByte(byte b, BOOL fWait);
extern void UARTPrintfStr(char *pString);
extern void UARTPrintfProgStr(PGM_P pString);
extern byte UARTReceiveByte(void);
extern void UARTSetBaudVal(uint bval);
extern void UARTInit(uint newBaudVal, BOOL fUSE_2X);
//extern void UARTInit(uint newBaudVal);
extern uint MapBaudVal2BaudRate(byte bVal);
extern void UARTSet115200(void);
inline BOOL UARTHasData(void)
{
	return ((GPIOR0 & _BV(fRcvdChar)) != 0);
}
inline BOOL IsUARTOutReady(void)
{
	return UARTOutReady;
}

