// Dutchtronix AVR Oscilloscope Clock
//
//  Copyright  2010 Johannes P.M. de Rie
//
//  All Rights Reserved
//
//  This file is part of the Dutchtronix Oscilloscope Clock Distribution.
//  Use, modification, and re-distribution is permitted subject to the
//  terms in the file named "LICENSE.TXT", which contains the full text
//  of the legal notices and should always accompany this Distribution.
//
//  This software is provided "AS IS" with NO WARRANTY OF ANY KIND.
//
//  This notice (including the copyright and warranty disclaimer)
//  must be included in all copies or derivations of this software.
//
// Firmware for Dutchtronix AVR Oscilloscope Clock
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>

#include <util/delay.h>

//#define BAUD 115200
#define BAUD 250000
#define BAUD_TOL 4
#include <util/setbaud.h>

#include "./config.h"
#include "./ClkISR.h"
#include "./sound.h"
#include "./sed1335.h"

#include "./monitor.h"

uint Timer3Counter;
byte Timer2Counter;
byte SecondsCnt;
unsigned long TickCnt;

char Welcome[] PROGMEM = "Welcome to Asteroids\r\n";

void timer1_init (void);
void timer2_init (void);

unsigned long GetTickCount(void)
{
	//One TickCnt is 16.66 mSecs
	return (unsigned long)((float)TickCnt * 16.66);
}

#define kDefaultAsteroidSize 18
#define kFrameDuration  35				// The Game Frame Rate (in milliseconds)
#define kMaxBullets     20              // The maximum on-screen bullets
#define kMaxAsteroids   10              // The maximum asteroids on-screen at once
#define kMinNewAsteroidInterval 600     // The minimum time interval (mS) to wait for a new asteroid to be added
#define kMaxNewAsteroidInterval 1200    // The maximum time interval (mS) to wait for a new asteroid to be added
#define kDemoWaitTime	30000			// wait time before demo mode starts
#define kFrameRedrawTime 5000			// time between frame redraws


#include "./Asteroids.c"

//#define UART_BAUD_RATE 115200			//use BAUD

void xGLCD_WriteChar(unsigned char ch)
{
}

int main(void)
{
	WDTimerOff();
	GPIOR0 = GPIOR1 = 0;
#if USE_2X
	Error. Does not work for 115,200
#endif
//	UARTInit((F_CPU + UART_BAUD_RATE * 8L) / (UART_BAUD_RATE * 16L) - 1, FALSE);
	UARTInit(UBRR_VALUE , USE_2X);

	InitSwitch();
	InitLed();
	InitTC0();
	timer1_init();
	timer2_init();
	S1.okToRepeat = S2.okToRepeat = TRUE;
	S3.okToRepeat = S4.okToRepeat = TRUE;
	InitTC3();

	xfunc_out = GLCD_WriteChar;

	GLCD_Initialize();
	GLCD_ClearGraphic();
	GLCD_ClearText();
	GLCD_TextGoTo(0,0);
	xprintf("%s\n", "Hello world ");
	sei();
//	UARTPrintfProgStr(Welcome);
//
//	Init random nr generator
//
	uint ee = eeprom_read_word(0);
	srand(ee);
	int nr = rand();
	eeprom_write_word(0,nr);
	Asteroid();
}
