; Terminal.asm	-- Source for Dutchtronix ScopeClock Terminal
;
;  Copyright  2008 Jan P.M. de Rie
;
;  All Rights Reserved
;
;  This file is part of the Dutchtronix Oscilloscope Clock Distribution.
;  Use, modification, and re-distribution is permitted subject to the
;  terms in the file named "LICENSE.TXT", which contains the full text
;  of the legal notices and should always accompany this Distribution.
;
;  This software is provided "AS IS" with NO WARRANTY OF ANY KIND.
;
;  This notice (including the copyright and warranty disclaimer)
;  must be included in all copies or derivations of this software.
;
; Terminal program for Dutchtronix AVR Oscilloscope Clock hardware
;
; General design
;
; Main:
;	Initialize
;	loop forever calling TerminalClockWorks and checking for
;		input such as the Push Button switch
;		or the Serial port
;
; TerminalClockWorks:
;	Refresh
;	Check various bits set by interrupt or other functions
;
; Register conventions
;
;	r8, r9 used as Grid Offset values (global)
;	r14 and r15 used as literals (global)
;	r25 used as a status register (global)
;	r11, r12 used as X and Y offsets in Refresh
;
;	functions usally preserve registers, except top level functions
;
; First Release: V3.0 (to stay in sync with the AVR Oscilloscope Clock)
; Minor Release: V3.0a Set the I2C lines to input when not in use. Save changed BaudRate in EEProm
; Second Release: V3.5 Merged with Dutchtronix AVR Oscilloscope Clock application
;
; Suggestions:
;				Auto shut-off display if no data received for 5 minutes or so
;				Screen saver if no data received (analog clock?)
;				Terminal Emulation
;
#define		MAXLINESIZE		20
#define		LINEVSIZE		18
#define		SCREENLINES		13
#define		TOPLINEVPOS		(((SCREENLINES)*LINEVSIZE)+3)


;
; Scope Terminal SRam Variables. Continue where Clock data ended.
;
	.dseg
;
; Pointer(s) to SRam Space based Text buffers.
; Not all are currently used.
;
.equ TERMSCANSSTRTABLEN =   15
TermScanTblStr0:		.byte	4		;terminal line
TermScanTblStr1:		.byte	4		;terminal line
TermScanTblStr2:		.byte	4		;terminal line
TermScanTblStr3:		.byte	4		;terminal line
TermScanTblStr4:		.byte	4		;terminal line
TermScanTblStr5:		.byte	4		;terminal line
TermScanTblStr6:		.byte	4		;terminal line
TermScanTblStr7:		.byte	4		;terminal line
TermScanTblStr8:		.byte	4		;terminal line
TermScanTblStr9:		.byte	4		;terminal line
TermScanTblStr10:		.byte	4		;terminal line
TermScanTblStr11:		.byte	4		;terminal line
TermScanTblStr12:		.byte	4		;terminal line
TermScanTblStrClk:		.byte	4		;Numeric Time Display
TermScanTbl_Str5:		.byte	4		;Chrono Display
;
; Terminal Screen Data Storage
; add one positon for NULL terminator plus one for Debug Marker
;
Line0:				.byte	(MAXLINESIZE+2)
Line1:				.byte	(MAXLINESIZE+2)
Line2:				.byte	(MAXLINESIZE+2)
Line3:				.byte	(MAXLINESIZE+2)
Line4:				.byte	(MAXLINESIZE+2)
Line5:				.byte	(MAXLINESIZE+2)
Line6:				.byte	(MAXLINESIZE+2)
Line7:				.byte	(MAXLINESIZE+2)
Line8:				.byte	(MAXLINESIZE+2)
Line9:				.byte	(MAXLINESIZE+2)
Line10:				.byte	(MAXLINESIZE+2)
Line11:				.byte	(MAXLINESIZE+2)
Line12:				.byte	(MAXLINESIZE+2)
Line13:				.byte	(MAXLINESIZE+2)

FlashBuffer:		.byte	(MAXLINESIZE+2)

CurLine:			.byte	1
CurPosition:		.byte	1
LineOverflow:		.byte	1

.equ		MAXBUNCHEDCHARS	=	128

;----------------------------------------------------------;
; Program code area
;
	.cseg

InitTerm:
	sts		CurLine,_0
	sts		CurPosition,_0			;beginning of line
	sts		LineOverflow,_0			;need this sometimes
	rcall	SetTermScreen			;initialize ScanTbl
	rcall	ClearTermScreen
	rcall	SetWelcomeTermScreen
	ret

RunTerm:
	cbi		GPIOR0,fLFPending
	cbi		GPIOR0,fCRPending
	rcall	TermSetNumDisplayBuf

#if 1
	ldiw	Z,SpecialTermClockWorks	;default func ptr
	stdw	fpSpecialClockWorks,Z
	sei
#endif

	call	UsrPrintNewLine
	call	TermShowStatus

;
; RunTerm Program Forever Loop.
; No registers need to be saved by the called functions
;
	rjmp	StartCursorTimer		;odd way to enter loop

TermMainLoop:
	rcall	TermClockWorks				;Update the display and process events
;
; Did we receive a Button Up Message (mode change)?
;
	sbrs	G_FLAGS,fUPS1
	rjmp	TermML500				;not UP
	cbr		G_FLAGS,(1<<fUPS1)
;
; Menu entered. Ignore incoming chars on the serial port
;
	CALL	MainMenuProcessing
	rcall	TermSetNumDisplayBuf		;in case something changed

#if MULTIAPP
;
; Did we change application in the menu? We are running Terminal
;
	lds		r16,CurrAppMode
	cpi		r16,APPCLOCK
	brne	TMSAVL100
	subi	r16,APPCLOCK			;rebase to 0
	rcall	UpdateStartApp_EEPprom	;r16 argument
	CALL	SetNumericDisplay
;	RCALL	SetChronoDispBuf		;should not be necessary
 	JMP		MainLoop
TMSAVL100:
	cpi		r16,APPGEN
	brne	TMSAVL200
	JMP		RunFuncGen
TMSAVL200:
	cpi		r16,APPBOOT
	brne	TMSAVL300
	rcall	InitiateSysReset			;restart
TMSAVL300:
#endif
TermML500:
;
; Was a char received on the UART
;
	sbrs	G_FLAGS,fRcvdChar  		;skip TermMainLoop jump is set
	rjmp	TestCursorTimer

ProcessBunchedChars:
	rcall	StopCursorFlashing
	ldi		r16,MAXBUNCHEDCHARS		;max chars per bunch
ProcessInp:
	push	r16
	call	TermSerialInput
	pop		r16
	sbrs	G_FLAGS,fRcvdChar  		;goto TermMainLoop if done
	rjmp	StartCursorTimer
	dec		r16
	brne	ProcessInp
;	rjmp	StartCursorTimer		;FALL THROUGH
;
; We want to show a blinking cursor at the current char position
; but only after no input was received for at least 1 second,
; to prevent wasting too much time setting up the flash line
;
StartCursorTimer:
	stsi	Time2Ticks,220
	rjmp	TermMainLoop

TestCursorTimer:
	lds		r16,Time2Ticks
	tst		r16
	brne	TermMainLoop
	rcall	StartCursorFlashing		;iff we're not flashing yet!
	rjmp	TermMainLoop	
;
; end of Main Program Loop
;

;
;	TermSerialInput
;
;	Accepts a buffered serial input char and update display memory
;
TermSerialInput:
#if DEBUG
	lddw	Z,CTextFlashPtr			;current Flash Ptr
	or		ZH,ZL					;must be NULL (explain)
	breq	TermSIL050
	rjmp	PC
TermSIL050:
#endif
	rcall	UART_ReceiveByte		;get char in r24
	cpi		r24,0x1F				;If not a control char branch to displayable char routine.
	brpl	TermSIL400				;could still be >127
	cpi		r24,BS					;Backspace
	brne	TermSIL60
	lds		r16,CurPosition
	tst		r16
	breq	TermSIL900				;done
	dec		r16
	sts		CurPosition,r16
	rjmp	TermSIL900				;done
TermSIL60:
	cpi		r24,CR					;If this is a carriage return character, 
	brne	TermSIL100
	sbic	GPIOR0,fCRPending		;is there one pending?
	rcall	DoTermLF				;yes, do LF
TermSIL75:
	sbis	GPIOR0,fLFPending		;is there an LF pending?
	rjmp	TermSIL90				;no, pend the CR
	lds		r24,LineOverflow		;was this a real LF?
	tst		r24
	breq	TermSIL900				;yes, ignore this CR
;
; we received a CR after a line overflow. Clear the line overflow status
;
	cbi		GPIOR0,fLFPending
	sts		LineOverflow,_0
TermSIL90:
	sbi		GPIOR0,fCRPending		;CR now pending
	rjmp	TermSIL900
TermSIL100:
	cpi		r24,TAB
	brne	TermSIL200
	ldi		r24,SPACE
	rjmp	TermSIL500
TermSIL200:
;
; Handle multiple LFs
; LF implies CR
; <CR><LF> is identical to <LF>
;
	cpi		r24,LF					;If its a linfeed char then set linefeed pending flag
	brne	TermSIL300
	sbic	GPIOR0,fLFPending		;check for multiple LFs
	rcall	DoTermLF
	sbi		GPIOR0,fLFPending
	cbi		GPIOR0,fCRPending		;clear
	rjmp	TermSIL900
TermSIL300:
;
; other control char. Display as ^+letter
;
	push	r24						;save
	ldi		r24,'^'
	rcall	DisplayableChar			;recurse
	pop		r24						;restore
	addi	r24,64					;map
	jmp		DisplayableChar
TermSIL400:
	sbrc	r14,7					;must be <=127
	rjmp	TermSIL900			;if not, ignore
;
; Displayable char routine
;
DisplayableChar:
TermSIL500:
;
;	If not a control char then do line feed if pending
;
	sbic	GPIOR0,fLFPending		;If linefeed is pending, then do the LF
	rcall	DoTermLF				;will reset CurPosition
	sbis	GPIOR0,fCRPending		;CR Pending?
	rjmp	TermSIL600				;no
	cbi		GPIOR0,fCRPending		;clear
	sts		CurPosition,_0			;to beginning of line
TermSIL600:
	rcall	GetTermLine				;get ptr to current line
	lds		r16,CurPosition			;positon within the line
	add		XL,r16
	adc		XH,_0
	st		X+,r24					;store char in Display Buffer
	inc		r16						;next
	sts		CurPosition,r16
	cpi		r16,MAXLINESIZE			;line overflow?
	brlt	TermSIL900				;no
	sbi		GPIOR0,fLFPending
	sts		LineOverflow,_1			;need this sometimes
TermSIL900:
	ret

DoTermLF:
	lds		r16,CurLine
	inc		r16						;next line
	cpi		r16,SCREENLINES
	brlt	DoTermLFL100
	rcall	ScrollTermScreen		;will clear last line
	ldi		r16,SCREENLINES-1			;last line
DoTermLFL100:
	sts		CurLine,r16				;save
	rcall	GetTermLine				;line ptr in X
	rcall	ClearTermLine			;clear new line
	sts		CurPosition,_0			;beginning of line
	cbi		GPIOR0,fLFPending		;clear
	cbi		GPIOR0,fCRPending		;clear
	sts		LineOverflow,_0			;clear
	ret								;Done

;
; Clear Terminal Data Line - fill it with spaces
;
; IN:	X -- line to clear
;
; OUT:  X -- ptr to next line
;
ClearTermLine:
	push	r16
	ldi		r16,MAXLINESIZE
	ldi		r17,SPACE
ClearTLL100:
	st		X+,r17
	dec		r16
	brne	ClearTLL100
	st		X+,_0				;terminator
	adiw	X,1					;skip debug marker
	pop		r16
	ret

ClearTermScreen:
	ldi		r16,SCREENLINES
	ldiw	X,Line0
ClearTCL100:
	rcall	ClearTermLine
	dec		r16
	brne	ClearTCL100
	ret

FillTermLine:
	push	r16
	ldi		r16,MAXLINESIZE
FillTLL100:
	st		X+,r17
	inc		r17
	sbrc	r17,7				;wrap at 128
	ldi		r17,' '
	dec		r16
	brne	FillTLL100
	st		X+,_0				;terminator
	adiw	X,1					;skip debug marker
	pop		r16
	ret

SetWelcomeTermScreen:
	ldiw	Z,MsgPszLogo*2		;Dutchtronix
	ldiw	X,Line0
	rcall	MsgCopyZToXStringZ
	ldiw	Z,TermMsgPszWelcome*2
	rcall	MsgCopyZToXStringZ
	mov		r16,_1
	sts		CurLine,r16				;start
	ldi		r16,SCREENLINES-8
	ldiw	X,Line8
	ldi		r17,'1'
SetWTCL300:
	rcall	FillTermLine
	dec		r16
	brne	SetWTCL300
	ret

SetTermScreen:
	ldi		r16,SCREENLINES
	ldiw	X,Line0
	ldiw	Z,TermScanTblStr0
	ldi		r17,TOPLINEVPOS
	ser		r18
	ldi		r19,7
SetTSL100:
	st		Z+,XL
	st		Z+,XH
	adiw	X,MAXLINESIZE+1			;ptr to debug marker
	st		X+,r18					;ptr to next line
	st		Z+,r19					;HPos
	st		Z+,r17					;VPos
	subi	r17,LINEVSIZE			;Line below
	dec		r16
	brne	SetTSL100
	ret
;
; Scroll One Line
;
ScrollTermScreen:
	ldiw	Z,TermScanTblStr0			;start here
	ld		r0,Z+					;save this line ptr
	ld		r1,Z+
	sbiw	Z,2						;back
	ldi		r17,SCREENLINES-1	;total - clock - first
	ldiw	X,TermScanTblStr1			;copy from here
ScrollTSL100:
	ld		r16,X+					;copy line ptr
	st		Z+,r16
	ld		r16,X+
	st		Z+,r16
	adiw	X,2						;both to next entry
	adiw	Z,2
	dec		r17						;ScanTbl counter
	brne	ScrollTSL100			;more
	st		Z+,r0					;store line ptr from 1st entry
	st		Z+,r1
	movw	XL,r0					;clear this line
	rcall	ClearTermLine
	ret
;
; GetTermLine:
;
;	get ptr to current char line
;
; OUT	X -- ptr to line
;		Z -- ScanTbl entry where line is used
;
GetTermLine:
	lds		r16,CurLine				;current line
	cpi		r16,SCREENLINES			;out of range?
	brlt	GetTLL100
	clr		r16						;reset
GetTLL100:
	lsl		r16						;times 4
	lsl		r16
	ldiw	Z,TermScanTblStr0			;ptr to first entry
	add		ZL,r16					;ptr to desired entry
	adc		ZH,_0
	ld		XL,Z+					;ptr to desired line
	ld		XH,Z+
	sbiw	Z,2						;ScanTbl Ptr where line is currently used
	ret
;
; Display a flashing cursor. This is done by building an alternate
; buffer for the current Display Line and mark the current line
; for flashing
;
StartCursorFlashing:
	lddw	Z,CTextFlashPtr			;current Flash Ptr
	mov		r16,ZL					;test for NULL
	or		r16,ZH
	brne	StartCFL800				;not NULL, already flashing
	rcall	GetTermLine				;get current line ptr in X
	ldiw	Z,FlashBuffer			;destination
StartCFL100:						;copy line
	ld		r16,X+					;get char
	st		Z+,r16					;copy, incl. NULL
	tst		r16						;test for NULL
	brne	StartCFL100
;
; compute current position in FlashBuffer
;
	ldiw	Z,FlashBuffer			;destination
	lds		r16,CurPosition			;current line ptr in X
	cpi		r16,MAXLINESIZE			;line overflow?
	brlt	StartCFL200				;no
	ldi		r16,MAXLINESIZE-1
	rjmp	StartCFL300
StartCFL200:
	sbis	GPIOR0,fLFPending		;if LF pending, cursor under last char
	rjmp	StartCFL300
	lds		r16,CurPosition			;don't go back too far
	tst		r16
	breq	StartCFL300
	dec		r16
	sts		CurPosition,r16
StartCFL300:
	add		ZL,r16
	adc		ZH,_0
	ldi		r16,UNDERSCORE
	st		Z,r16
	rcall	GetTermLine				;ScanTbl ptr in Z,current line ptr in X
	movw	r0,XL					;swap them
	movw	XL,ZL
	movw	ZL,r0
	ldiw	A,FlashBuffer			;interface
	movw	r0,r16
	ldi		r16,0xff				;forever
	rcall	AddFlashItem
	stdw	CTextFlashPtr,X			;save Flash Ptr
	ret
StartCFL800:
	ret

StopCursorFlashing:
	lddw	X,CTextFlashPtr			;current Flash Ptr
	rcall	RemoveFlashItem
	clrdw	CTextFlashPtr
	ret
;
; Workhorse function. Does not need to preserve registers
; 	
TermClockWorks:
#if CHRONOMETER
	lds		r0,ChronoEnabled
	tst		r0
	breq	TClockWL100
	lds		r16,RefreshCnt
	andi	r16,0b00000111		;every 8 refresh cycles
	brne	TClockWL100
	CALL	UpdateChronoBuf		;not always
TClockWL100:
#endif
	rcall	TermScreenRefresh		;update the display. No need to preserve registers
	CALL	CommonClockWorks
	ret

;
; Interface function to TermClockWorks
;	Preserves r16,r24,Z

SpecialTermClockWorks:
	push	r16					;temp register
	push	r24
	pushw	Z
	rcall	TermClockWorks
	popw	Z
	pop		r24
	pop		r16
	ret

;
; TermScreenRefresh: draw one image on the CRT
;
; r17 used as the ScanTbl index, pointing to next entry to process
; Z: current ScanTbl entry pointer
;
; No registers saved by this function. This is the
; responsibility of the caller.
;
.def	XOffset			=	r11
.def	YOffset			=	r12
;
#if 0
; Reminder:
.def	_0			= r15	;Zero register
.def	_1			= r14	;One register
.def	G_FLAGS 	= r25	;Global Flags
.def	GXOffset	= r8
.def	GYOffset	= r9
#endif
;
TermScreenRefresh:
;
; Process SRam based String Data
;
	ldiw	X,TermScanTblStr0		;First Pointer
	ldi		r17,TERMSCANSSTRTABLEN	;count
	CALL	TextTableScanSRam
#if CHRONOMETER
	ldsw	X,RefreshCnt
	adiw	X,1
	stsw	RefreshCnt,X
#endif
	ret

TermSetNumDisplayBuf:
	ldiw	X,TermScanTblStrClk	;Update Scan Table
	ldiw	Z,NumDispBuf	;needed twice
	st		X+,ZL
	st		X+,ZH
	CALL	ComputeSRamHPos	;center
	st		X+,r16			;Horizontal Position
	ldi		r16,TERMNUMERICVPOS	;Vertical Position
	st		X+,r16
	ret

TermShowStatus:
	ldiw	Z,MsgPszLogo*2				;Dutchtronix
	rcall	Special_UARTPS
	ldiw	Z,TermMsgPszSignOn*2	;signon
	rcall	Special_UARTPS
	ldi		r24,' '
	rcall	Special_UARTSB
	ldiw	Z,MsgPszVersion*2			;V3.x
	rcall	Special_UARTPS
	rcall	UsrPrintNewLine
	rcall	UsrShowStatus
	ret

TermMsgPszSignOn:			.db	" AVR Oscilloscope "
TermMsgPszWelcome:			.db	" Terminal",0


