; gps.asm - included source for Dutchtronix Oscilloscope Clock
;
;  Copyright  2008 Johannes P.M. de Rie
;
;  All Rights Reserved
;
;  This file is part of the Dutchtronix Oscilloscope Clock Distribution.
;  Use, modification, and re-distribution is permitted subject to the
;  terms in the file named "LICENSE.TXT", which contains the full text
;  of the legal notices and should always accompany this Distribution.
;
;  This software is provided "AS IS" with NO WARRANTY OF ANY KIND.
;
;  This notice (including the copyright and warranty disclaimer)
;  must be included in all copies or derivations of this software.
;
; Firmware for Dutchtronix AVR Oscilloscope Clock
;
; NMEA processing
;
; Always enabled? SERIALIO off? Dynamic change?
;
#if 0

At End Of Line, Parse line. If time info, retrieve current time/date.
 Update AVRTime/AVRDate if different
Watch potential sync issues. NMEA output is delayed.
Add internal 10 msec tickCounter (0..99), synchronized with PPS. If Delta < margin, don't update AVRTime
Option: display internal tickCounter in 24Hr Numeric Display. Need another timer.
Debugging NMEA a challenge. Feed NMEA lines from term. program.
Set Debug Console also to 4800 bps for output
#endif
;
; GPS related code.
; Memory use is wasteful but need to reduce beam pausing
;
; Parsing the record in the UART buffer is an option but
; care must be taken to not lose the next incoming record,
; which may be another NMEA type which we don't want but need
; to discard properly.
; Worst case, double buffering may be needed, but hopefully we can
; just use the RXLFDetected flag carefully.
;
#if GPS

#define GPSStatusField 2
#define GPSTimeField 1
#define GPSDateField 9

;
; While in Change or Menu (??) mode, ignore incoming GPS records
;
DiscardGPSInput:
;
; Check if a LF was detected in the UART input buffering.
; This method requires that the input buffer is large enough
; to hold a complete NMEA record
;
	lds		r16,RXLFDetected	;complete buffer received?
	tst		r16
	breq	DiscardGPSL100		;no, done
#if DEBUG
;
; verify that GPS processing is enabled
;
	lds		r0,GPSInEnabled		;are we processing GPS info
	sbrs	r0,0				;skip if ok
	rjmp	PC
#endif
	CALL	UARTFlushRecord		;just dump it
	sts		RXLFDetected,_0		;clear record complete flag
DiscardGPSL100:
	ret

;
; While in normal Clock mode, process incoming GPS records
;
CheckGPSInput:
;
; Check if a LF was detected in the UART input buffering.
; This method requires that the input buffer is large enough
; to hold a complete NMEA record
;
	lds		r16,RXLFDetected	;complete buffer received?
	tst		r16
	breq	CheckGPSL800		;no, done
#if DEBUG
;
; verify that GPS processing is enabled
;
	lds		r0,GPSInEnabled		;are we processing GPS info
	sbrs	r0,0				;skip if ok
	rjmp	PC
#endif
#if 1
	lds		r16,GPSOKToUpdate	;time to process GPS record?
	tst		r16
	breq	DiscardGPSInput		;dump it and return to caller
#if FUNMODES
	CALL	TestPlayModes		;doing any Fun Mode?
	tst		r16
	brne	DiscardGPSInput		;yes: dump it and return to caller
#endif		;FUNMODES
	rcall	ProcessGPSRecord
#else
	lds		r16,GPSOKToUpdate	;time to process GPS record?
	tst		r16
	breq	CheckGPSL700
#if FUNMODES
	CALL	TestPlayModes		;doing any Fun Mode?
	tst		r16
	brne	CheckGPSL700		;yes, ignore
#endif		;FUNMODES
	rcall	ProcessGPSRecord
	rjmp	CheckGPSL800
CheckGPSL700:
	CALL	UARTFlushRecord		;dump it
;	sts		RXLFDetected,_0		;clear record complete flag. Done in FlushRecord
#endif
CheckGPSL800:
	ret

ProcessGPSRecord:
;
; new records may come in while we copy the current record
;
	sts		RXLFDetected,_0		;clear record complete flag
	ldi		r19,GPSInBufLength-1 ;buffer size
	ldiw	X,GPSInBuf			;compute buffer ptr
	CALL	UART_ReceiveLine
#if 1		;Current Method
#if DEBUG
;	rcall	GPSRecFeedback
#endif

;
; Parse GPS line. This is just as good a time as any
;
; Look for "$GPRMC" record
;
ProcessGPSL300:
	ldiw	X,GPSInBuf			;buffer ptr: NMEA Id
	ldiw	Z,NMMEA_GPRMC*2
	CALL	StrCmp_S_F			;compare Sram and Flash based string
	sbrs	r16,0					;result
	rjmp	ProcessGPSL900			;no match
;
; found "$GPRMC" record
;
#endif
ProcessGPSL310:

	ldi		r16,GPSStatusField	;locate Status substr
	CALL	GPSInBufLocate		;result in X
	ld		r16,X				;check for valid time flag
	cpi		r16,'A'
	rjne	ProcessGPSL900
;
; 'A' flag found. We have a valid record. Show this on the clock
;
;
; Clear TimeToProcess flag and start timer
;
	sts		GPSOKToUpdate,_0	;do not process GPS record
	RCALL	StartTC1Countdown4Sec	;shared with Demo Mode
;
; Show a non-flashing G for 1 second.
; Set the "G" in the warning field (Use the Flash system in non-flashing mode)
; Make it flash if we update the time or date 
; Be careful setting it again since it may still be flashing. Just
; skip in that case.
; Use the LED control () to bypass this "valid record received" indicator.
;
	lds		r16,LedDisabled			;if LedDisabled TRUE, bypass "G" indicator
	tst		r16	
	brne	ProcessGPSL320
	ldi		r16,FLASHPERSEC*1		;1 second
	rcall	GPSStartNoFlashing	
ProcessGPSL320:
	ldi		r16,GPSTimeField	;locate Time substr
	CALL	GPSInBufLocate		;result in X
	rcall	GPSGetBinVal
	std		UTCHrs,r24
	rcall	GPSGetBinVal
	std		UTCMins,r24
	rcall	GPSGetBinVal
	std		UTCSecs,r24
;
; get UTC date
;
	ldi		r16,GPSDateField	;locate Date substr
	CALL	GPSInBufLocate		;result in X
	rcall	GPSGetBinVal
	std		UTCDay,r24
	rcall	GPSGetBinVal
	std		UTCMonth,r24
	rcall	GPSGetBinVal
	std		UTCYear,r24
;
; Convert UTC time to local time.
; The time zone offset needs to be entered by the user.
; If Automatic Daylight Saving Time (DST) is active, the
; clock will do the correction (for the supported years)
;
; We only support Hour offsets.
;
; Note that this conversion may include a change to the current date,
; including possibly the current year, which in turn may affect the
; DST computations. Since this year change can only happen around
; 12/31 - 1/1, DST will not be active.
;
	ldd		r16,UTCHrs			;time we just received
	lds		r0,GPSOffset		;value -12..+12
	add		r16,r0				;new time
	std		UTCHrs,r16
;	tst		r16
	brlt	ProcessGPSL400		;to previous day
	cpi		r16,24				;to next day?
	brsh	ProcessGPSL450		;yes
	rjmp	ProcessGPSL500
ProcessGPSL400:					;retreat date one day
	subi	r16,-24				;add 24 to correct the hour
	std		UTCHrs,r16
	CALL	UTCDecDate			;TEST
	rjmp	ProcessGPSL500
ProcessGPSL450:					;advance date one day
	subi	r16,24				;subtract 24 to correct the hour
	std		UTCHrs,r16
	CALL	UTCIncDate			;TEST
ProcessGPSL500:
;
; Now check if a Daylight Saving Time adjustment of 1 hour needs to be applied
; to the UTC time/date variables
;
	rcall	DSTGPSInWindow
	brcs	ProcessGPL550			;not in window
	CALL	UTCAdvanceTimeOneHr
ProcessGPL550:
#if DEBUG
	sts		GPSUpdateRTCDate,_0
	sts		GPSUpdateRTCTime,_0
#endif

;
;	Compare UTC time and AVR time
;
	ldd		r16,UTCHrs
	ldd		r0,AvrHrs
	cp		r16,r0
	brne	ProcessGPSL600				;Update Time	
	ldd		r16,UTCMins
	ldd		r0,AvrMins
	cp		r16,r0
	brne	ProcessGPSL600				;Update Time
	ldd		r16,UTCSecs
	ldd		r0,AvrSecs
	cp		r16,r0
	brne	ProcessGPSL575				;Update Time
	rjmp	ProcessGPSL650				;same
;
; Time differs in seconds only.
;	Optional refinement:
;		look at TimeTicks which tracks the current second in 1/100 increments.
;		If the AvrTime is within 90%
;
ProcessGPSL575:
;	if UTCsecs == AvrSecs+1 and (TimeTicks > 90) no adjustment
;	ldd		r16,TimeTicks
;	if UTCsecs == AvrSecs-1 and (TimeTicks < 10) no adjustment
;	ldd		r16,TimeTicks

; Update AVR Time
;
ProcessGPSL600:
	ldd		r0,UTCHrs
	std		AvrHrs,r0
	ldd		r0,UTCMins
	std		AvrMins,r0
	ldd		r0,UTCSecs
	std		AvrSecs,r0
#if DEBUG
	sts		GPSUpdateRTCTime,_1
#endif
;
; Update the RTC with the new time
;
	CALL	RTCSetTime					;update RTC
	std		DisplayRedrawFlg,_1
ProcessGPSL640:
;
;	Compare UTC date AVR date
;
ProcessGPSL650:
	ldd		r16,UTCYear
	ldd		r0,AvrYear
	cp		r16,r0
	brne	ProcessGPSL700				;Update Date
	ldd		r16,UTCMonth
	ldd		r0,AvrMonth
	cp		r16,r0
	brne	ProcessGPSL700				;Update Date
	ldd		r16,UTCDay
	ldd		r0,AvrDay
	cp		r16,r0
	brne	ProcessGPSL700				;Update Date
	rjmp	ProcessGPSL750				;same
;
; Update AVR date
;
ProcessGPSL700:
	ldd		r0,UTCYear
	std		AvrYear,r0
	ldd		r0,UTCMonth
	std		AvrMonth,r0
	ldd		r0,UTCDay
	std		AvrDay,r0
#if DEBUG
	sts		GPSUpdateRTCDate,_1
#endif
;
; Update the RTC with the new date.
;
	CALL	RTCSetDate			;update RTC
	std		DisplayRedrawFlg,_1
ProcessGPSL750:
#if DEBUG
;
; send UTC info to serial port. Only available in Debug Mode
;
	CALL	DisplayUTCDateTime
	sts		GPSUpdateRTCDate,_0
	sts		GPSUpdateRTCTime,_0
#endif
;
; Using the DisplayRedrawFlg is somewhat flawed
; since it won't get cleared until the next seconds transition.
; We shouldn't get any interesting NMEA records until then though
;
	ldd		r0,DisplayRedrawFlg		;Date or Time updated?
	tst		r0
	breq	ProcessGPSL900			;no
;
; Inform the DST subsystem
;
	rcall	DSTRetroNotApply		;no fix but set bitvector flags
;
; Show that time/date has been updated from GPS input
;
	ldiw	X,ScanTblStatG
	ldiw	Z,CharG*2				;'G'
	CALL	CheckFlashItem			;Flash Tbl Ptr in X
	mov		r0,XL					;check ptr
	or		r0,XH
	breq	ProcessGPSL850			;not found, add item
;
; HACK, change FlashTbl in place
;
	ldi		r16,5					;point to duration field
	add		XL,r16
	adc		XH,_0
	ldi		r16,FLASHPERSEC*5		;5 seconds
	st		X+,r16					;update flashcount
	adiw	X,2						;point to alternate vector tbl ptr
	st		X+,_0					;set alternate vector tbl ptr to NULL (flash)
	st		X+,_0
	rjmp	ProcessGPSL900
ProcessGPSL850:
	ldi		r16,FLASHPERSEC*5		;5 seconds
	rcall	GPSStartFlashing
ProcessGPSL900:
	ret
;
; Display a flashing 'G' for n seconds
;
; IN r16 -- number of seconds to flash
;
GPSStartFlashing:
	ldiw	X,ScanTblStatG
	ldiw	Z,CharG*2				;'G'
	CALL	AddFlashItemNoAlt		;Flash Tbl Ptr in X
	ret
;
; Display a steady 'G' for n seconds
;
; IN r16 -- number of seconds to flash
;
; The "G" may still be flashing from the last
; GPS correction. Skip the action in that case.
;
GPSStartNoFlashing:
	ldiw	X,ScanTblStatG
	ldiw	Z,CharG*2				;'G'
	CALL	CheckFlashItem			;Flash Tbl Ptr in X
	mov		r0,XL					;check ptr
	or		r0,XH
	brne	GPSSNFL200				;skip if present
	ldiw	X,ScanTblStatG			;reload X. Z still good
	ldi		r16,low(CharG*2)		;use 'G' again for
	mov		r0,r16					;alt ptr -> steady
	ldi		r16,high(CharG*2)
	mov		r1,r16
	ldi		r16,1*FLASHPERSEC		;1 second
	CALL	AddFlashItem			;Flash Tbl Ptr in X
GPSSNFL200:
	ret
;
; Increment UTC Date
;
UTCIncDate:
	ldiw	Z,UTCTimeDateBlk
	CALL	IncDatefromZ
	ret
;
; Decrement UTC Date
;
UTCDecDate:
	ldiw	Z,UTCTimeDateBlk
	CALL	DecDatefromZ
	ret

UTCAdvanceTimeOneHr:
	ldd		r17,UTCHrs		;update Hours
	inc		r17
	cpi		r17,24			;check for overflow
	brlt	UTCAdvanceTOH900
	std		UTCHrs,_0
	rcall	UTCIncDate
	clr		r17				;new UTCHrs
UTCAdvanceTOH900:
	std		UTCHrs,r17
	ret

; IN: r16 day to check
;
; OUT: r16 == 0 error, else input value
;
CheckUTCDaysInMonth:
	ldiw	Z,UTCTimeDateBlk
	CALL	GetDaysInMonthfromZ		;in r17
	inc		r17
	cp		r16,r17
	brlo	CheckUTCDIML100
	clr		r16
CheckUTCDIML100:
	ret

;
; compute number of days present in UTCMonth.
;
; OUT: r17 number of days in UTCMonth
;
GetUTCDaysInMonth:
	pushw	Z	
	ldiw	Z,UTCTimeDateBlk
	CALL	GetDaysInMonthfromZ
	popw	Z	
	ret
;
; Locate a substr in the GPS NMEA record,
; The record is terminated by LF (guaranteed)
;
GPSInBufLocate:
	ldiw	X,GPSInBuf
	clr		r17					;debug check
	mov		r0,r16				;field cnt
	inc		r0					;it's 0 based
GPSPIBL100:
	dec		r0					;correct field #?
	breq	GPSPIBL900
;
; skip one field
;
GPSPIBL200:
	ld		r16,X+				;get char
	inc		r17					;debug count
	cpi		r16,LF				;end of record?
	breq	GPSPIBL900			;done
#if DEBUG
	cpi		r17,GPSInBufLength	;limit
	brcc	GPSPIBL900			;end of buffer
#endif
	cpi		r16,','				;field separator?
	breq	GPSPIBL100			;yes, X points to next field
	rjmp	GPSPIBL200
GPSPIBL900:
	ret
;
;	Compare SRam based string (X) with Flash based String (Z)
;	',' is termination char
;
;	result in r16
;
;	uses r16,r0
;
StrCmp_S_F:
	ld		r16,X+
	lpm		r0,Z+				;from (Z) to r0
	cp		r16,r0
	brne	StrCmpL100			;not equal, one of them could be ','
	cpi		r16,','				;equal and end of string?
	brne	StrCmp_S_F			;no
	mov		r16,_1				;equal
	rjmp	StrCmpL200
StrCmpL100:
	clr		r16
StrCmpL200:
	ret

NMMEA_GPRMC:
	.db		"$GPRMC,",0
#if DEBUG
;
; Display Current Date and Time based on UTC Variables
; Create the text in the Transmit Buffer, then send
;
; Buffer overflow checked at code time (fixed format)
;
DisplayUTCDateTime:
	pushw	Z					;needed?
	pushw	X
	ldiw	X,TXBuffer
	ldi     r24,'G'
	st		X+,r24
	ldi     r24,'P'
	st		X+,r24
	ldi     r24,'S'
	st		X+,r24
	ldi     r24,':'
	st		X+,r24
	CALL	FormatUTCDate
	ldi     r24,' '
	st		X+,r24
	rcall	FormatUTCTime
	lds		r0,GPSUpdateRTCTime
	tst		r0
	breq	DUDTL200
	ldi		r24,'T'				;show time was updated
	st		X+,r24
DUDTL200:
	lds		r0,GPSUpdateRTCDate
	tst		r0
	breq	DUDTL300
	ldi		r24,'D'				;show date was updated
	st		X+,r24
DUDTL300:
	ldi     r24,13
	st		X+,r24
	ldi     r24,10
	st		X+,r24
	st		X+,_0				;Termination Char
	CALL	Special_UARTTX		;start Transmitting
	popw	X
	popw	Z					;needed?
	ret

;
; Format UTC Date Time in Buffer (X)
;
; Buffer overflow checked at code time (fixed format)
;
FormatUTCDate:
	ldiw	Z,UTCTimeDateBlk
	CALL	FormatDatefromZ
	ret
;
; Format UTC Time in Transmit Buffer (X)
;
; Buffer overflow checked at code time (fixed format)
;
; Duplicate code. See UpdClockNum
;
FormatUTCTime:
	ldiw	Z,UTCTimeDateBlk
	CALL	FormatTimefromZ
	ret
#endif		;DEBUG
	
GPSGetBinVal:
	ld		r16,X+				;high digit
	ld		r17,X+				;low digit
;
; convert 2-byte ASCII to 1-byte BCD
;
	swap	r16
	andi	r16,0b11110000
	andi	r17,0b00001111
	or		r16,r17
	mov		r24,r16
	CALL	bcdtobin
	ret

#if SERIALIO
;
; Toggle the mode of the Serial IN port between
; commands and NMEA records
; This includes changing the baudrate of the serial port.
; Used by command line interpreter
;
GPSToggleMode:
	lds		r0,GPSInEnabled
	tst		r0
	brne	GPSTML100
;
;	Enable GPS input mode
;
	rcall	GPSOnBaudRate
	rjmp	GPSTML200
GPSTML100:
	rcall	GPSOffBaudRate
GPSTML200:
	eor		r0,_1			;toggle
	sts		GPSInEnabled,r0
;
; save mode of Serial port in EEProm
;
	mov		r16,r0
	CALL	UpdateGPS_EEprom
;
; Save New Baud Rate in EEprom
;
	lds		r16,BaudVal
	rcall	UpdateBaudrate_EEPprom
	ret
#endif

GPSOnBaudRate:
#if MULTIBAUD
	ldi		r16,(BAUDVAL4800-BAUDVALBASE)	;GPS uses 4800 BPS by default
	sts		BaudVal,r16
	rcall	SetBaudVal
#else
#if SERIALIO
	CALL	UARTFlushPendingOutput		;included in SetBaudVal
#endif
	CALL	UART_Set4800
#endif
	ret

GPSOffBaudRate:
#if MULTIBAUD
	ldi		r16,(BAUDVAL19200-BAUDVALBASE)	;Clock uses 19200 BPS by default
	sts		BaudVal,r16
	rcall	SetBaudVal
#else
#if SERIALIO
	CALL	UARTFlushPendingOutput		;included in SetBaudVal
#endif
	CALL	UART_Set19200
#endif
	ret

	
#endif		;GPS
