; rtc.asm - included source for Dutchtronix Oscilloscope Clock
;
;  Copyright  2008 Johannes P.M. de Rie
;
;  All Rights Reserved
;
;  This file is part of the Dutchtronix Oscilloscope Clock Distribution.
;  Use, modification, and re-distribution is permitted subject to the
;  terms in the file named "LICENSE.TXT", which contains the full text
;  of the legal notices and should always accompany this Distribution.
;
;  This software is provided "AS IS" with NO WARRANTY OF ANY KIND.
;
;  This notice (including the copyright and warranty disclaimer)
;  must be included in all copies or derivations of this software.
;
; Firmware for Dutchtronix AVR Oscilloscope Clock
;
.EQU	DS1307ID	=	0b11010000
.EQU	PCF8563ID	=	0b10100010

#if ds1370
.equ	RTCID	=	DS1307ID
#else
.equ	RTCID	=	PCF8563ID
#endif

#define DODELAY	rcall I2CDELAY
;
; RTC Routines
;
#if ds1370
InitRTCInt:
	pushw	A
	ldi		r17,RTCCNT			;set SQWE and 01 value
	ldi		r16,0b10010000
	rcall	I2CRndWrt
	popw	A
	ret

StartRTCInt:
	ret

ResetRTCInt:
	ret

StopRTCInt:
	ret

StartRTCClockOut:
	ret

StopRTCClockOut:
	ret

#else
InitRTCInt:
	pushw	A
	rcall	StopRTCClockOut
	rcall	StartRTCInt
	popw	A
	ret

StartRTCInt:
	ldi		r17,RTCCNT			;set countdown value
	ldi		r16,64
	rcall	I2CRndWrt
	ldi		r17,RTCTCR			;update Timer/Control Register
	ldi		r16,0b10000001		;enable TE, set 64 Hz source Clock
	rcall	I2CRndWrt
	ldi		r17,RTCCS2			;update Control/Status 2
	ldi		r16,0b00010001		;clear TF, set TI/TP set TIE
	rcall	I2CRndWrt
	ret

ResetRTCInt:
;
; reload Countdown, clear TF flag in RTC
; Apparently it is not necessary to reload these timer values
; after a timer interrupt.
;
	ldi		r17,RTCCNT
	ldi		r16,64
	rcall	I2CRndWrt
	ldi		r17,RTCCS2			;update Control/Status 2
	ldi		r16,0b00010001		;clear TF, set TIE
	rcall	I2CRndWrt
	ret
;
; Note ClockOut is Open Drain Signal
;
StartRTCClockOut:
	ldi		r17,RTCCLK
	ldi		r16,0b10000000		;enable ClockOut at 32.768 KHz
	rcall	I2CRndWrt
	ret

StopRTCClockOut:
	ldi		r17,RTCCLK
	clr		r16					;disable ClockOut (power saver)
	rcall	I2CRndWrt
	ret

#endif

I2CDELAY:
	nop
	nop
	nop
	nop
	nop
	nop
#if (Mega168 | Mega328p)
	nop
	nop
#endif
	ret

I2CInit:
;
; Configure the I2C pins for the RTC.
; Currently configures as OUTPUT. Adding pull-up resistors to an output port
; seems useless.
;
; Configure these RTC pins as input (default), possibly with pull-ups,
; When accessing the RTC, change the direction as needed, leaving
; them as input again on exit.
; This way, we can write to these bits when accessing the DAC without
; affecting the RTC in any way
;
	sbi     PORT_RTC,BIT_SDA
	sbi     PORT_RTC,BIT_SCL
	ret
;
; RANDOM READ:	Address in r17	Data returned in r16 and r24
;	No other registers used
;
I2CRndRd:
	push	r19
	clr		r19
I2CRRdL100:
	dec		r19			;max 256 tries
	brne	I2CRRdL150	;still good
;
; failed
;
	sts		RTCNotPresent,_1
	rjmp	I2CRRdL200
I2CRRdL150:
	breq	I2CRRdL200
	rcall	I2CStart
	DODELAY
	ldi		r16,RTCID|0	;address|write command to set RTC Register Pointer
	rcall	I2CTX
	DODELAY
	rcall	I2CAck		;wait acknowledge
	brcs	I2CRRdL100
	mov		r16,r17		;address high
	rcall	I2CTX
	DODELAY
	rcall	I2CAck		;wait acknowledge
	brcs	I2CRRdL100
	rcall	I2CStop
;
	rcall	I2CStart
	ldi		r16,RTCID|1	;address|read command
	rcall	I2CTX
	DODELAY
	rcall	I2CAck
	brcs	I2CRRdL100
	rcall	I2CRX
	sbi		PORT_RTC,BIT_SDA	;NOT acknowledge
	sbi		PORT_RTC,BIT_SCL
I2CRRdL200:
	rcall	I2CStop
	mov		r24,r16
	pop		r19
	ret
;
;  RANDOM WRITE: Address in r17	Data to write in r16
;	No other registers used
;
I2CRndWrt:
	push	r19
	clr		r19
I2CRWrL100:
	dec		r19			;max 256 tries
	brne	I2CRWrL150	;still good
;
; failed
;
	sts		RTCNotPresent,_1
	rjmp	I2CRWrL200
I2CRWrL150:
	rcall	I2CStart
	DODELAY
	push	r16
	ldi		r16,RTCID|0	;address|write command to set RTC Register Pointer
	rcall	I2CTX
	DODELAY
	rcall	I2CAck		;wait acknowledge
	pop		r16
	brcs	I2CRWrL100
	push	r16
	mov		r16,r17		;address high
	rcall	I2CTX
	rcall	I2CAck		;wait acknowledge
	pop		r16
	brcs	I2CRWrL100
	push	r16
	rcall	I2CTX
	rcall	I2CAck		;wait acknowledge
	pop		r16			;Data to write
	brcs	I2CRWrL100
	nop
I2CRWrL200:
	rcall	I2CStop
	pop		r19
	ret
;
; CHECK BIT I2C_ACKNOWLEDGE BIT
;	CARRY = 1: FAIL 
;	CARRY = 0: PASS 
;
I2CAck:
	cbi		DDR_RTC,BIT_SDA    ;BIT_SDA input port
	sbi		PORT_RTC,BIT_SDA
	sec
	sbi		PORT_RTC,BIT_SCL
	DODELAY
	sbis	PIN_RTC,BIT_SDA
	clc
	cbi		PORT_RTC,BIT_SCL
	sbi		DDR_RTC,BIT_SDA   ;BIT_SDA output port
	ret
;
;   Transmit 1 byte to RTC
;
I2CTX:
	push	r19
	ldi		r19,8
I2CTX1:
	sbi		PORT_RTC,BIT_SDA
	lsl		r16
	brcs	I2CTX2
	cbi		PORT_RTC,BIT_SDA
I2CTX2:
	sbi		PORT_RTC,BIT_SCL
	DODELAY
	cbi		PORT_RTC,BIT_SCL
	dec		r19
	brne	I2CTX1
	pop		r19
	ret
;
;   Receive 1 byte from RTC 
;
I2CRX:
	push	r19
	cbi		DDR_RTC,BIT_SDA
	sbi		PORT_RTC,BIT_SDA
	clr		r16
	ldi		r19,8
I2CRX1:
	sbi		PORT_RTC,BIT_SCL
	sec
	DODELAY
	sbis	PIN_RTC,BIT_SDA
	clc
	rol		r16
	cbi		PORT_RTC,BIT_SCL
	DODELAY
	dec		r19
	brne	I2CRX1
	sbi		DDR_RTC,BIT_SDA	
	pop		r19
	ret

I2CStop:
	cbi		PORT_RTC,BIT_SDA
	sbi		PORT_RTC,BIT_SCL	;stop condition
	nop
#if (Mega168 | Mega328p)
	nop							;critical
#endif
	sbi		PORT_RTC,BIT_SDA
;
; Configure the I2C bits as inputs
;
	nop
	cbi     DDR_RTC,BIT_SDA		;
	cbi     DDR_RTC,BIT_SCL 
	ret

I2CStart:
;
; Configure the I2C bits for output
;
	sbi     DDR_RTC,BIT_SDA		;
	sbi     DDR_RTC,BIT_SCL 

	sbi		PORT_RTC,BIT_SDA
	sbi		PORT_RTC,BIT_SCL	;start condition
	nop
#if (Mega168 | Mega328p)
	nop
#endif
	cbi		PORT_RTC,BIT_SDA
	nop
#if (Mega168 | Mega328p)
	nop
#endif
	cbi		PORT_RTC,BIT_SCL	;critical
#if (Mega168 | Mega328p)
	nop
#endif
	ret
;
; Update Time Variable AvrSecs
;
; only called when syncing with the RTC. Does NOT call "BatteryWarning"
;
RTCReadSecs:
;
; update AvrSecs from RTC
; uses r16, r17
;
	push	r24
	ldi     r17,RTCSEC
	rcall   I2CRndRd
	andi	r24,0b01111111
	rcall	bcdtobin		;convert
	cpi		r24,60
	brlt	RTCRSL100
	clr		r24
RTCRSL100:
	std		AvrSecs,r24
	pop		r24
	ret
;
; Update Time Variable AvrSecs, AvrMins, AvrHrs
;
RTCReadTime:
;
; get time units in r24 in BCD form
; uses r16, r17
;
	push	r24
	ldi     r17,RTCSEC
	rcall   I2CRndRd
	sbrc	r24,7			;if VL bit is on, call BatteryWarning TEST
	rcall	BatteryWarning
	andi	r24,0b01111111
	rcall	bcdtobin		;convert
	cpi		r24,60
	brlt	RTCRTL100
	clr		r24
RTCRTL100:
	std		AvrSecs,r24
	ldi     r17,RTCMIN
	rcall   I2CRndRd
	andi	r24,0b01111111
	rcall	bcdtobin		;convert
	cpi		r24,60
	brlt	RTCRT_L200
	clr		r24
RTCRT_L200:
	std		AvrMins,r24
	ldi     r17,RTCHOUR
	rcall   I2CRndRd
	andi	r24,0b00111111
	rcall	bcdtobin		;convert
	cpi		r24,24
	brlt	RTCRT_L300
	clr		r24
RTCRT_L300:
	std		AvrHrs,r24
	pop		r24
	ret
;
; Update Date Variable AvrDay, AvrMonth, AvrYear
;
; get date units in r24 in BCD form from RTC
; uses r16, r17
;
RTCReadDate:
	push	r16				;test if needed
	push	r24
	ldi     r17,RTCDAY
	rcall   I2CRndRd
	andi	r24,0b00111111
	rcall	bcdtobin		;convert
	cpi		r24,32
	brlt	RTCReadDateL100
	mov		r24,_1
RTCReadDateL100:
	std		AvrDay,r24
	ldi     r17,RTCMONTH
	rcall   I2CRndRd
	andi	r24,0b00011111
	rcall	bcdtobin		;convert
	tst		r24				;test for 0 month
	breq	RTCReadDate_L150
	cpi		r24,12+1
	brlt	RTCReadDate_L200
;
; r24 value is too small or too large
;
RTCReadDate_L150:
	mov		r24,_1
RTCReadDate_L200:
	std		AvrMonth,r24
	ldi     r17,RTCYEAR
	rcall   I2CRndRd
	rcall	bcdtobin		;convert
	cpi		r24,100
	brlt	RTCReadDate_L300
	clr		r24
RTCReadDate_L300:
	std		AvrYear,r24
	pop		r24
	pop		r16				;test if needed
	ret
;
; Update RTC with data from AvrSecs, AvrMins, AvrHrs
;
;
RTCSetTime:
#if DEMOMODE
	lds		r24,DemoEnabled
	tst		r24
	brne	RTC_WU100
#endif
	ldd		r24,AvrSecs
	rcall	RTCSetSecs
	ldd		r24,AvrMins
	rcall	RTCSetMins
	ldd		r24,AvrHrs
	rcall	RTCSetHours
RTC_WU100:
	ret
;
; Set RTC Time. Binary input in r24
; Uses r16,r17
;
RTCSetSecs:
	push	r24
	mov		r16,r24
	cpi		r16,60				;sanity check
	brsh	RTC_S100
	std		AvrSecs,r24
	rcall	bintobcd
	ldi		ah,RTCSEC			;seconds
	rcall	I2CRndWrt
RTC_S100:
	pop		r24
	ret

RTCSetMins:
	mov		r16,r24
	cpi		r16,60				;sanity check
	brsh	RTC_Mi100
	std		AvrMins,r24
	rcall	bintobcd
	ldi		ah,RTCMIN			;minutes
	rcall	I2CRndWrt
RTC_Mi100:
	ret

RTCSetHours:
	mov		r16,r24
	cpi		r16,24				;sanity check
	brsh	RTC_H100
	std		AvrHrs,r24
	rcall	bintobcd
	ldi		ah,RTCHOUR			;hours
	rcall	I2CRndWrt
RTC_H100:
	ret
;
; Set RTC Day
;
; IN:  r16 is Day
;
; OUT: r16==0 means invalid Day
; else: AvrDay updated
;
RTCSetDay:
	rcall	CheckAVRDaysInMonth	;r16 input
	tst		r16					;check result
	breq	RTC_D100			;invalid
	std		AvrDay,r16
	mov		r24,r16
	rcall	bintobcd
	ldi		ah,RTCDAY			;day
	rcall	I2CRndWrt
RTC_D100:
	ret

RTCSetMonth:
	mov		r16,r24
	cpi		r16,13				;sanity check
	brsh	RTC_Mo100
	std		AvrMonth,r24
	rcall	bintobcd
	ldi		ah,RTCMONTH		;month
	rcall	I2CRndWrt
RTC_Mo100:
	ret

RTCSetYear:
	mov		r16,r24
	std		AvrYear,r24
	rcall	bintobcd
	ldi		ah,RTCYEAR			;year
	rcall	I2CRndWrt
	ret

RTCSetDate:
	ldd		r16,AvrDay
	rcall	RTCSetDay				;will validate Day
	tst		r16
	brne	RTCSDL200				;ok
	rcall	RTCReadDate				;restore AVR date from RTC, fall through
RTCSDL200:
	ldd		r24,AvrMonth
	rcall	RTCSetMonth
	ldd		r24,AvrYear
	rcall	RTCSetYear
	ret
;
; set time to 00:00:00 and date to 01/01/2000
;
CleanInitRTC:
	clr		r24
	rcall	RTCSetSecs
	clr		r24
	rcall	RTCSetMins
	clr		r24
	rcall	RTCSetHours
	ldi		r16,1
	rcall	RTCSetDay
	ldi		r24,1
	rcall	RTCSetMonth
	clr		r24
	rcall	RTCSetYear
	rcall	UpdateRTCINIT_EEprom	;mark as initialized
	ret
