//
//  AVR Oscilloscope Clock Display Tables Generator, originally written 
//	 by Sascha Ittner ((c) 2004 GPL).
//
// Updated for Dutchtronix AVR Oscilloscope clock by Jan de Rie 2007-2008
//
// X,Y coordinates start in lower left corner (0,0)
//
// Make sure that the ESCAPE char is never generated in the vector tables
// 
#include <stdio.h>
#include <math.h>

#define OLDDIAL 1
#define SIMPLEXFONT 0
#define INVERT(x) (256 - (x))

#define M_PI 3.14159265358979323846
#define COUNT 0
#define LIMITED_SCALE 0
#define DIGITAL_TIME 1
#define NUMERICVPOS 81
#define NUMERICSECOFFSET 72
#define NUMERICMINOFFSET 52
#define NUMERICHROFFSET 20
#define MINNUMERICH 6
#define MINNUMERICV 8
#define DEFCHARWIDTH 10
#define NUMERICCOLON1OFFSET 30
#define NUMERICCOLON2OFFSET 62
#define CLOCKDIGITVPOS 0
#define SIGVPOS 160
#define SIGHPOS 192
#define DEMOVPOS 150
#define DEMOHPOS 192
#define STAT1VPOS 192
#define	STAT1HPOS 128
#define LETTERA ('A'-'A')
#define LETTERB ('B'-'A')
#define LETTERC ('C'-'A')
#define LETTERD ('D'-'A')
#define LETTERE ('E'-'A')
#define LETTERF ('F'-'A')
#define LETTERG ('G'-'A')
#define LETTERH ('H'-'A')
#define LETTERI ('I'-'A')
#define LETTERJ ('J'-'A')
#define LETTERK ('K'-'A')
#define LETTERL ('L'-'A')
#define LETTERM ('M'-'A')
#define LETTERN ('N'-'A')
#define LETTERO ('O'-'A')
#define LETTERP ('P'-'A')
#define LETTERQ ('Q'-'A')
#define LETTERR ('R'-'A')
#define LETTERS ('S'-'A')
#define LETTERT ('T'-'A')
#define LETTERU ('U'-'A')
#define LETTERV ('V'-'A')
#define LETTERW ('W'-'A')
#define LETTERX ('X'-'A')
#define LETTERY ('Y'-'A')
#define LETTERZ ('Z'-'A')

#define	DOT		1
#define	SLASH	2
#define	COLON	3
#define DASH	4
#define SPACE	5
//
// BLANK is a wide version of SPACE
//
#define BLANK	6

#if COUNT
int XValues[256], YValues[256];
int XYPair[65536];
int PairCnt = 0;
int AddrCnt = 0;
void RecordXYPair(int x, int y)
{
	XValues[x]++;
	YValues[y]++;
	XYPair[(x << 8) + y]++;
	PairCnt++;
}
void DisplayXYPairs()
{
	int i,j,n=0;
	for (i = 0; i < 65536; i += 256) {
		for (j = 0; j < 256; j++) {
			if (XYPair[i+j]) {
				printf("(0x%2.2x,0x%2.2x, 0x%2.2x) ", i>>8, j, XYPair[i+j]);
				n++;
				if ((n %8) == 0) printf("\n");
			}
		}
	}
	printf("\n");
}

#endif

void MakeScale(char *name, float r, float len1, float len2, float len3);
void MakeDotDial(char *name, float r);
void MakeSimplePtr(char *name, float r1);
void MakeTriangPtr(char *name, float r1, float r2);
void MakeNum(char *name, int x, int y, int height, int width, int val, int term, int force, int fEnabled);
void MakeNum3(char *name, int x, int y, int val, int term, int force, int fEnabled);
void MakeLetter(char *name, int x, int y, int height, int width, int val, int term, int force, int fEnabled);
int	 MakeNewLetter(char *name, int x, int y, int height, int width, char ch, int term, int force, int fEnabled);
void MakeLetterLow(char *name, int x, int y, int height, int width, int val, int term, int force, int fEnabled);
void MakeChar(char *name, int x, int y, int height, int width, int val, int term, int force, int fEnabled);
void MakeRomanLetter(char *name, int x, int y, int val, int term, int force, int fEnabled);
void MakeCalibrationData(char *pName);

int NumCount = 0;	// force even number of bytes per line

int main(void) {
  int i, letter, xOffset, savexOffset;
  char str_buf[256];
  float r = 105.0, a;
  int dig, x1, y1;

#if COUNT
	int xCnt, yCnt, xyCnt, xyFreqCnt, xyFreqTotalCnt;
	for (i = 0; i < 256; i++) {
		XValues[i] = 0;
		YValues[i] = 0;
	}
	for (i = 0; i < 65536; i++) {
		XYPair[i] = 0;
	}
#endif

  printf(";\n");
  printf("; Data for Dutchtronix Oscilloscope Clock	\n");
  printf(";\n");
  printf("\n");

  MakeScale("DialData:",126,5,10,12);
  printf("\n");
#if 0
  MakeDotDial("DialDots:", 126);
  printf("\n");
#endif
//
// Regular Dial Numbers
//
  for(dig=0; dig<60; dig += 5) {
    a=((float)(dig*6))/180*M_PI;
    x1=(int)(128+sin(a)*r);
	y1=(int)(128-cos(a)*r);
	switch (dig) {
		case 0:  MakeNum3("DialDigits:",x1-6, INVERT(30), 1,0, 0, 1);
				 MakeNum3(NULL         ,x1+6, INVERT(30), 2,0, 0, 1);
				 break;
		case 5:  MakeNum3(NULL		 , x1+1, INVERT(y1),  (dig / 5),0, 1, 1);
				 break;
		case 10: MakeNum3(NULL		 , x1-2, INVERT(y1),  (dig / 5),0, 1, 1); break;
		case 15: MakeNum3(NULL        ,   x1, INVERT(y1) , 3,0, 0, 1);
				 break;
		case 20: MakeNum3(NULL		 ,   x1, INVERT(y1),  (dig / 5),0, 1, 1); break;
		case 25: MakeNum3(NULL		 , x1-2, INVERT(y1-2),  (dig / 5),0, 1, 1); break;
		case 30: MakeNum3(NULL        ,   x1, INVERT(y1)+3 , 6,0, 0, 1);
				 break;
		case 35: MakeNum3(NULL		 , x1+2, INVERT(y1-2),  (dig / 5),0, 1, 1); break;
		case 40: MakeNum3(NULL		 ,  x1,  INVERT(y1) , (dig / 5),0, 1, 1); break;
		case 45: MakeNum3(NULL        ,   x1, INVERT(y1) , 9,0, 1, 1);
				 break;
		case 50: MakeNum3(NULL		 , x1-4, INVERT(y1) ,            1,0, 1, 1); 
				 MakeNum3(NULL		 , x1+8, INVERT(y1) , 		   0,0, 1, 1); break;
		case 55: MakeNum3(NULL		 , x1-2, INVERT(y1) , 		   1,0, 1, 1);
				 MakeNum3(NULL		 , x1+8, INVERT(y1) , 		   1,0xff, 1, 1); break;
	}
  }
//
// 24 Hrs format Dial Numbers
//
  for(dig=0; dig<60; dig += 5) {
    a=((float)(dig*6))/180*M_PI;
    x1=(int)(128+sin(a)*r);
	y1=INVERT((int)(128-cos(a)*r));
	switch (dig) {
		case 0:  MakeNum("DialDigits24:",x1-7, y1,   3,3, 2,0, 0, 1);
				 MakeNum(NULL           ,x1+7, y1,   3,3, 4,0, 0, 1); break;
		case 5:  MakeNum(NULL		    ,x1-10, y1-2,3,3, 1,0, 1, 1);
				 MakeNum(NULL		    ,x1+2, y1-2, 3,3, 3,0, 1, 1);break;
		case 10: MakeNum(NULL		    ,x1-10, y1-2,3,3, 1,0, 1, 1);
				 MakeNum(NULL		    ,x1+2, y1-2, 3,3, 4,0, 1, 1); break;
		case 15: MakeNum(NULL           ,x1-12,y1,   3,3, 1,0, 0, 1);
				 MakeNum(NULL           ,x1, y1,     3,3, 5,0, 0, 1); break;
		case 20: MakeNum(NULL		    ,x1-10, y1,  3,3, 1,0, 1, 1);
				 MakeNum(NULL		    ,x1+2, y1,   3,3, 6,0, 1, 1); break;
		case 25: MakeNum(NULL		    ,x1-10,y1-2, 3,3, 1,0, 1, 1);
				 MakeNum(NULL		    ,x1+2, y1-2, 3,3, 7,0, 1, 1); break;
		case 30: MakeNum(NULL           ,x1-4, y1,   3,3, 1,0, 0, 1);
				 MakeNum(NULL           ,x1+7, y1,   3,3, 8,0, 0, 1); break;
		case 35: MakeNum(NULL		    ,x1-2,  y1,  3,3, 1,0, 1, 1);
				 MakeNum(NULL		    ,x1+10, y1,  3,3, 9,0, 1, 1); break;
		case 40: MakeNum(NULL		    ,x1-1,  y1,  3,3, 2,0, 1, 1);
				 MakeNum(NULL		    ,x1+16, y1,  3,3, 0,0, 1, 1); break;
		case 45: MakeNum(NULL           ,x1   , y1,  3,3, 2,0, 1, 1);
				 MakeNum(NULL           ,x1+12, y1,  3,3, 1,0, 1, 1); break;
		case 50: MakeNum(NULL		    ,x1-1,  y1,  3,3, 2,0, 1, 1); 
				 MakeNum(NULL		    ,x1+16, y1,  3,3, 2,0, 1, 1); break;
		case 55: MakeNum(NULL		    ,x1-5, y1-4, 3,3, 2,0, 1, 1);
				 MakeNum(NULL		    ,x1+12,y1-4, 3,3, 3,0xff, 1, 1); break;
	}
  }
  printf("\n");
//
// Roman Numerals Dial
//
  for(dig=0; dig<60; dig += 5) {
    a=((float)(dig*6))/180*M_PI;
    x1=(int)(128+sin(a)*r);
	y1=(int)(128-cos(a)*r);
	switch (dig) {
		case 0:  MakeRomanLetter("DialDigitsRoman:",x1,INVERT(y1 /* 30 */), 12 ,0, 1, 1);		//XII
				 break;
		case 5:  MakeRomanLetter(NULL		 , x1-3, INVERT(y1),  (dig / 5),0, 1, 1);
				 break;
		case 10: MakeRomanLetter(NULL		 , x1-2, INVERT(y1),  (dig / 5),0, 1, 1); break;
		case 15: MakeRomanLetter(NULL        , x1-3, INVERT(y1) , 3,0, 0, 1);
				 break;
		case 20: MakeRomanLetter(NULL		 , x1-6, INVERT(y1),  (dig / 5),0, 1, 1); break;
		case 25: MakeRomanLetter(NULL		 , x1-3, INVERT(y1)+6,  (dig / 5),0, 1, 1); break;
		case 30: MakeRomanLetter(NULL        ,   x1, INVERT(y1)+6 , 6,0, 0, 1);
				 break;
		case 35: MakeRomanLetter(NULL		 , x1+3, INVERT(y1)+6,  (dig / 5),0, 1, 1); break;
		case 40: MakeRomanLetter(NULL		 , x1+6,  INVERT(y1) , (dig / 5),0, 1, 1); break;
		case 45: MakeRomanLetter(NULL        , x1+3, INVERT(y1) , (dig / 5),0, 1, 1); break;
		case 50: MakeRomanLetter(NULL		 , x1+6, INVERT(y1) , (dig / 5),0, 1, 1); break;
		case 55: MakeRomanLetter(NULL		 , x1+6, INVERT(y1) , (dig / 5),0xff, 1, 1); break;
	}
  }
//
// Minimal Dial
//
	for(dig=0; dig<60; dig += 5) {
	    a=((float)(dig*6))/180*M_PI;
		x1=(int)(128+sin(a)*r);
		y1=(int)(128-cos(a)*r);
		switch (dig) {
			case 0:  MakeNum3("DialDigitsMin:	",x1-6, INVERT(30), 1,0, 0, 1);
					 MakeNum3(NULL       ,x1+6, INVERT(30), 2,0, 0, 1);
					 break;
			case 15: MakeNum3(NULL       ,x1, INVERT(y1) , 3,0, 0, 1);
					 break;
			case 30: MakeNum3(NULL       ,x1, INVERT(y1)+3 , 6,0, 0, 1);
					 break;
			case 45: MakeNum3(NULL       ,x1, INVERT(y1) , 9,0xff, 1, 1);
					 break;
		}
	}


#if !OLDDIAL
  for(dig=0; dig<60; dig += 5) {
    a=((float)(dig*6))/180*M_PI;
    x1=(int)(128+sin(a)*r);
	y1=INVERT((int)(128-cos(a)*r));
	switch (dig) {
		case 0:  //MakeNum("DialDigits:",x1-6, (30),2, 2, '1',0, 0, 1);
				 //MakeNum(NULL         ,x1+6, (30),2, 2, '2',0, 0, 1);
				 break;
		case 5:  MakeNewLetter("DialDigits:", x1-10, (y1-10), 2, 2, '1',0, 1, 1);
				 break;
		case 10: MakeNewLetter(NULL		 , x1-13, (y1-10), 2, 2, '2',0, 1, 1); break;
		case 15: //MakeNum(NULL        ,   x1, (y1) ,3,2, 2,0, 0, 1);
				 break;
		case 20: MakeNewLetter(NULL		 , x1-8, (y1-4), 2, 2, '4',0, 1, 1); break;
		case 25: MakeNewLetter(NULL		 , x1-8, (y1-4), 2, 2, '5',0, 1, 1); break;
		case 30: //MakeNum(NULL        ,   x1, (y1)+3 ,3,3, 6,0, 0, 1);
				 break;
		case 35: MakeNewLetter(NULL		 , x1-4, (y1-4), 2, 2, '7',0, 1, 1); break;
		case 40: MakeNewLetter(NULL		 ,  x1-8,  (y1-4) ,2, 2, '8',0, 1, 1); break;
		case 45: //MakeNum(NULL        ,   x1, (y1) ,3,3, 9,0, 1, 1);
				 break;
		case 50: MakeNewLetter(NULL		 , x1-10, (y1-10) , 2, 2,'1',0, 1, 1); 
				 MakeNewLetter(NULL		 , x1+2, (y1-10) , 2, 2,'0',0, 1, 1); break;
		case 55: MakeNewLetter(NULL		 , x1-8, (y1-10) , 2, 2,'1',0, 1, 1);
				 MakeNewLetter(NULL		 , x1+4, (y1-10) , 2, 2,'1',0xff, 1, 1); break;
	}
  }
#endif


  printf("\n");
 
  MakeSimplePtr("SecPtrData:",108);
  MakeTriangPtr("MinPtrData:",102,2);
  MakeTriangPtr("HrPtrData:",60,2);

#if COUNT
	xCnt=0;
	yCnt=0;
	xyCnt = 0;
	xyFreqCnt = 0;
	xyFreqTotalCnt = 0;
	for (i = 0; i < 256; i++) {
		if (XValues[i]) ++xCnt;
		if (YValues[i]) ++yCnt;
	}
	for (i = 0; i < 65536; i++) {
		if (XYPair[i]) xyCnt++;
		if (XYPair[i] >= 8) {
			xyFreqTotalCnt += XYPair[i];
			xyFreqCnt++;
		}
	}
	DisplayXYPairs();
	printf("xCnt: %d, yCnt: %d, PairCnt: %d, xyCnt: %d, AddrCnt: %d, FreqCnt: %d FreqTot: %d\n", xCnt,yCnt, PairCnt, xyCnt, AddrCnt, xyFreqCnt, xyFreqTotalCnt);
#endif

  return 0;
}

void MakeScale(char *name, float r, float len1, float len2, float len3) {
  int min;
  float r2;
  float a;
  int x1;
  int y1;
  int x2;
  int y2;
  int i;

  i=0;
#if LIMITED_SCALE
//
// skip minor markers
//
  for(min=0; min<60; min=min+5) {
#else
  for(min=0; min<60; min=min+1) {
#endif
    a=((float)(min*6))/180*M_PI;

    r2=r;
    if ((min % 15) == 0) {
      r2=r2-len3;
    } else if ((min % 5) == 0) {
//      r2=r2-len2;
      r2=r2-len2;
    } else {
      r2=r2-len1;
    }

    x1=(int)(128+sin(a)*r);
    y1=INVERT((int)(128-cos(a)*r));
    x2=(int)(128+sin(a)*r2);
    y2=INVERT((int)(128-cos(a)*r2));

    if (i == 0) {
      if ((min == 0) && (name != NULL)) {
        printf("%-15s .db ",name);
      } else {
        printf("                .db ");
      }
    } else {
      printf(",");
    }

    printf("0x%2.2x,0x%2.2x,0x%2.2x,0x%2.2x",x1,y1,x2,y2);
#if COUNT
	RecordXYPair(x1, y1);
	RecordXYPair(x2, y2);
#endif
#if LIMITED_SCALE
    if (min == 55) {
#else
    if (min == 59) {
#endif
      printf(",0xff");
    } else {
      printf(",0x00");
    }

    if ((i++) == 3) {
      printf("\n");
      i=0;
    }
  }
  printf("\n");
}

void MakeDotDial(char *name, float r) {
  int min;
  float a;
  int x1;
  int y1;
  int i;

  printf("#if BEAMPARK\n");

  i=0;
  for(min=0; min<60; min=min+1) {
    a=((float)(min*6))/180*M_PI;

    x1=(int)(128+sin(a)*r);
    y1=INVERT((int)(128-cos(a)*r));

    if (i == 0) {
      if ((min == 0) && (name != NULL)) {
        printf("%-15s .db ",name);
      } else {
        printf("                .db ");
      }
    } else {
      printf(",");
    }

    printf("0x%2.2x,0x%2.2x",x1,y1);
#if COUNT
	RecordXYPair(x1, y1);
#endif
#if 0
	if (min == 59) {
      printf(",0xff");
    } else {
      printf(",0x00");
    }
#endif
    if ((i++) == 7) {
      printf("\n");
      i=0;
    }
  }
  printf("\n");
  printf("#endif\n");
}

void MakeSimplePtr(char *name, float r1) {
  int sec;
//  int len;
  float a;
  int x;
  int y;
  int i;

  i=0;
  for(sec=0; sec<60; sec++) {


    a=((float)(sec*6))/180*M_PI;
    x=(int)(128+sin(a)*r1);
    y=256 - (int)(128-cos(a)*r1);
	//
	// make sure pointer for 0, 15, 30,45 is straight
	//
	switch (sec) {
		case 0:
		case 30: x = 0x80; break;
		case 15:
		case 45: y = 0x80; break;
	}



    if (i == 0) {
      if ((sec == 0) && (name != NULL)) {
        printf("%-15s .db ",name);
      } else {
        printf("                .db ");
      }
    } else {
      printf(",");
    }

    printf("0x%2.2x,0x%2.2x,0x80,0x80,0xff",x,y);
#if COUNT
	RecordXYPair(x, y);
	RecordXYPair(128, 128);
#endif

    if ((i++) == 3) {
      printf("\n");
      i=0;
    }
  }
  printf("\n");
}

void MakeTriangPtr(char *name, float r1, float r2) {
  int min;
//  int len;
  float a;
  int x1;
  int y1;
  int x2;
  int y2;
  int x3;
  int y3;
  int i;

  i=0;
  for(min=0; min<60; min=min+1) {

    a=((float)(min*6))/180*M_PI;

    x1=(int)(128+sin(a+(M_PI/2))*r2);
    y1=INVERT((int)(128-cos(a+(M_PI/2))*r2));
    x2=(int)(128+sin(a)*r1);
    y2=INVERT((int)(128-cos(a)*r1));
    x3=(int)(128+sin(a-(M_PI/2))*r2);
    y3=INVERT((int)(128-cos(a-(M_PI/2))*r2));

    if (i == 0) {
      if ((min == 0) && (name != NULL)) {
        printf("%-15s .db ",name);
      } else {
        printf("                .db ");
      }
    } else {
      printf(",");
    }

	printf("0x%2.2x,0x%2.2x,0x%2.2x,0x%2.2x,0x%2.2x,0x%2.2x,0x%2.2x,0x%2.2x,0xff",x1,y1,x2,y2,x3,y3,x1,y1);
#if COUNT
	RecordXYPair(x1, y1);
	RecordXYPair(x2, y2);
	RecordXYPair(x3, y2);
#endif

    if ((i++) == 1) {
      printf("\n");
      i=0;
    }
  }
  printf("\n");
}

void MakeNum(char *name, int x, int y, int height, int width, int val, int term, int force, int fEnabled) {
  int i;
  int n;
  int c1[]={0,3,0,-3,255};		// 1
  int c2[]={-2,2,-1,3,1,3,2,2,2,1,-2,-3,2,-3,255};
  int c3[]={-2,2,-1,3,1,3,2,2,2,1,1,0,2,-1,2,-2,1,-3,-1,-3,-2,-2,-255,-1,0,1,0,255};
  int c4[]={2,0,-2,0,1,3,1,-3,255};
  int c5[]={2,3,-2,3,-2,0,1,0,2,-1,2,-2,1,-3,-1,-3,-2,-2,255};
  int c6[]={2, 2,1, 3,-1, 3,-2, 2,-2,-2,-1,-3,1,-3,2,-2,2,-1,1,0,-1,0,-2,-1,255};
  int c7[]={-2,3, 2,3, 2,2, 1,1, 0,-1, 0,-3,255};
  int c8[]={-1,0,-2,1,-2,2,-1,3,1,3,2,2,2,1,1,0,-1,0,-2,-1,-2,-2,-1,-3,1,-3,2,-2,2,-1,1,0,255};
  int c9[]={2,1,1,0,1,0,-1,0,-2,1,-2,2,-1,3,1,3,2,2,2,-2,1,-3,-1,-3,-2,-2,255};
  int c0[]={-2,2,-1,3,1,3,2,2,2,-2,1,-3,-1,-3,-2,-2,-2,2,255};
  int *c[]={c0,c1,c2,c3,c4,c5,c6,c7,c8,c9};
  int cx;
  int cy;

  if (val>9) return;

  if (name != NULL) {
    printf("%-15s .db ",name);
  } else {
		if ((NumCount % 2) == 0) {
			printf("                .db ");
		}
  }

  i=0;
  n=0;
  while ((cx = c[val][i]) != 255) {
    i++;
    if (cx == -255) {
      printf("0x00");
	  NumCount++;
      n=n+1;
    } else {
      cy=c[val][i];
      i++;

      cx=(cx*width)+x;
      if (cx<1)
		  cx=1;
      if (cx>254) cx=254;
      cy=(cy*height)+y;
      if (cy<1)   cy=1;
      if (cy>254) cy=254;

      printf("0x%2.2x,0x%2.2x",cx,cy);
	  NumCount += 2;
#if COUNT
	if (fEnabled)
		RecordXYPair(cx, cy);
#endif
	  n=n+2;
    }
      printf(",");
  }
  printf("0x%2.2x",term);
  NumCount++;
  if (force || ((NumCount % 2) == 0)) {
	  if (force && ((NumCount %2) == 1)) {
		  printf(",0x00");
	  }
	  printf("\n");
	  NumCount = 0;
	} else {
	printf(",");
  }

}

void MakeNum3(char *name, int x, int y, int val, int term, int force, int fEnabled) {
  int i;
  int n;
  int c1[]={0,9,0,-9,255};		// 1
//  int c2[]={-6,6,-3,9,-255, -1,9,3,9,-255, 5,7,6,6,-255, 6,4,6,3,-255, 4,1,-6,-9,-255, -4,-9, 6,-9,255};
  int c2[]={-6,6,-3,9,3,9,6,6,6,3,-6,-9,6,-9,255};
  int c3[]={-6,6,-3,9,3,9,6,6,6,3,3,0,6,-3,6,-6,3,-9,-3,-9,-6,-6,-255,-3,0,3,0,255};
  int c4[]={6,0,-6,0,3,9,3,-9,255};
  int c5[]={6,9,-6,9,-6,0,3,0,6,-3,6,-6,3,-9,-3,-9,-6,-6,255};
  int c6[]={6, 6,3, 9,-3, 9,-6, 6,-6,-6,-3,-9,3,-9,6,-6,6,-3,3,0,-3,0,-6,-3,255};
  int c7[]={-6,9, 6,9, 6,6, 3,3, 0,-3, 0,-9,255};
  int c8[]={-3,0,-6,3,-6,6,-3,9,3,9,6,6,6,3,3,0,-3,0,-6,-3,-6,-6,-3,-9,3,-9,6,-6,6,-3,3,0,255};
  int c9[]={6,3,3,0,3,0,-3,0,-6,3,-6,6,-3,9,3,9,6,6,6,-6,3,-9,-3,-9,-6,-6,255};
  int c0[]={-6,6,-3,9,3,9,6,6,6,-6,3,-9,-3,-9,-6,-6,-6,6,255};
  int *c[]={c0,c1,c2,c3,c4,c5,c6,c7,c8,c9};
  int cx;
  int cy;

  if (val>9) return;

  if (name != NULL) {
    printf("%-15s .db ",name);
  } else {
		if ((NumCount % 2) == 0) {
			printf("                .db ");
		}
  }

  i=0;
  n=0;
  while ((cx = c[val][i]) != 255) {
    i++;
    if (cx == -255) {
      printf("0x00");
	  NumCount++;
      n=n+1;
    } else {
      cy=c[val][i];
      i++;

      cx=(cx)+x;
      if (cx<1)
		  cx=1;
      if (cx>254) cx=254;
      cy=(cy)+y;
      if (cy<1)   cy=1;
      if (cy>254) cy=254;

      printf("0x%2.2x,0x%2.2x",cx,cy);
	  NumCount += 2;
#if COUNT
	if (fEnabled)
		RecordXYPair(cx, cy);
#endif
	  n=n+2;
    }
      printf(",");
  }
  printf("0x%2.2x",term);
  NumCount++;
  if (force || ((NumCount % 2) == 0)) {
	  if (force && ((NumCount %2) == 1)) {
		  printf(",0x00");
	  }
	  printf("\n");
	  NumCount = 0;
	} else {
	printf(",");
  }

}

void MakeLetter(char *name, int x, int y, int height, int width, int val, int term, int force, int fEnabled)
{
	int i;
	int n;
	int c0[]={255};
	int c1[]={-2,-3,-2,3,0,1,2,3,2,-3,255};							//M
	int c2[]={-2,-3,-2,3,1,3,2,2,2,1,1,0,-2,0,255};					//P
	int c3[]={-2,2,-2,3,2,3,2,-3,-2,-3,-2,-2,255};					//J
	int c4[]={-2,-3,0,3,2,-3,-255,1,-1,-1,-1,255};					//A
	int c5[]={-2,-3,-2,3,1,3,2,2,2,1,1,0,-2,0,-255,1,0,2,-1,2,-3,255};//R
	int c6[]={-2,-3,-2,3,1,3,2,2,2,1,1,0,-2,0,1,0,2,-1,2,-2,1,-3,-2,-3,255};//B
	int	c7[]={-1,-3, 1,-3, -255, 0,-3, 0,3, -255, -1,3, 1,3, 255};	//I
	int	c8[]={2,2,1,3,-1,3,-2,2,-2,-2,-1,-3,1,-3,2,-2, 255};		//C
	int	c9[]={2,3,-2,3,-2,-3,-255,-2,0,1,0, 255};					//F
	int	c10[]={-2,3, 1,3, 2,2, 2,-2, 1,-3, -2,-3, -2,3, 255};		//D
	int	c11[]={2,3, -2,3, -2,-3, 2,-3, -255, 1,0, -2,0, 255};		//E
	int c12[]={2,2,1,3,-1,3,-2,2,-2,-2,-1,-3,1,-3,2,-2,2,-1,0,-1,255};	//G
	int c13[]={-2,3,-2,-3,-255,-2,0,2,0,-255,2,-3,2,3,255};			//H
	int c14[]={-2,3,-2,-3,-255,2,3,0,0,-2,0,-255,-1,0,2,-3,255};	//K
	int	c15[]={-2,3,-2,-3,2,-3,255};								//L
	int c16[]={-2,-3,-2,3,2,-3,2,3,255};							//N
	int c17[]={-2,2,-1,3,1,3,2,2,2,-2,1,-3,-1,-3,-2,-2,-2,2,255};	//O
	int c18[]={2,-2,0,-3,-1,-3,-2,-2,-2,2,-1,3,1,3,2,2,2,-2,-255,0,-1,2,-3,255};	//Q
	int c19[]={2,2,1,3,-1,3,-2,2,-2,1,-1,0,1,0,2,-1,2,-2,1,-3,-1,-3,-2,-2,255}; //S
	int c20[]={-2,3,2,3,-255,0,3,0,-3,255};							//T
	int c21[]={-2,3,-2,-2,-1,-3,1,-3,2,-2,-255,2,3,2,-3,255};		//U
	int c22[]={-2,3,0,-3,2,3,255};									//V
	int c23[]={-2,3,-1,-3,0,-1,1,-3,2,3,255};						//W
	int c24[]={-2,3,2,-3,-255,2,3,-2,-3,255};						//X
	int c25[]={-2,3,-2,1,-1,0,1,0,2,1,-255,2,3,2,-2,1,-3,-1,-3,-2,-2,255}; //Y
	int c26[]={-2,3,2,3,-2,-3,2,-3,255};							//Z

//	            A  B  C  D   E   F  G   H   I  J  K   L   M  N   O   P  Q   R  S   T   U   V   W   X   Y   Z
	int *c[26]={c4,c6,c8,c10,c11,c9,c12,c13,c7,c3,c14,c15,c1,c16,c17,c2,c18,c5,c19,c20,c21,c22,c23,c24,c25,c26};
	int cx;
	int cy;

  if (val>25) return;

  if (name != NULL) {
    printf("%-15s .db ",name);
  } else {
		if ((NumCount % 2) == 0) {
			printf("                .db ");
		}
  }

  i=0;
  n=0;
  while ((cx = c[val][i]) != 255) {
    i++;
    if (cx == -255) {
      printf("0x00");
	  NumCount++;
      n=n+1;
    } else {
      cy=c[val][i];
      i++;

      cx=(cx*width)+x;
      if (cx<1)   cx=1;
      if (cx>254) cx=254;
      cy=(cy*height)+y;
      if (cy<1)   cy=1;
      if (cy>254) cy=254;

      printf("0x%2.2x,0x%2.2x",cx,cy);
	  NumCount += 2;
#if COUNT
	if (fEnabled)
		RecordXYPair(cx, cy);
#endif
	  n=n+2;
    }
      printf(",");
  }
  printf("0x%2.2x",term);
  NumCount++;
  if (force || ((NumCount % 2) == 0)) {
	  if (force && ((NumCount %2) == 1)) {
		  printf(",0x00");
	  }
	  printf("\n");
	  NumCount = 0;
	} else {
	printf(",");
  }
}

void MakeLetterLow(char *name, int x, int y, int height, int width, int val, int term, int force, int fEnabled)
{
	int i;
	int n;
	int	c0[]={255};
	int	c4[]={-1,-3,-1,-1,2,-1,2,-3,-1,-3,-255,2,-3,2,3,255};		//d
//			  A  B  C  D  E  F  G  H  I  J  K  L  M  N  O  P  Q  R  S  T  U  V  W  X  Y  Z
	int *c[]={c0,c0,c0,c4,c0,c0,c0,c0,c0,c0,c0,c0,c0,c0,c0,c0,c0,c0,c0,c0,c0,c0,c0,c0,c0,c0};
	int cx;
	int cy;

  if (val>25) return;

  if (name != NULL) {
    printf("%-15s .db ",name);
  } else {
		if ((NumCount % 2) == 0) {
			printf("                .db ");
		}
  }

  i=0;
  n=0;
  while ((cx = c[val][i]) != 255) {
    i++;
    if (cx == -255) {
      printf("0x00");
	  NumCount++;
      n=n+1;
    } else {
      cy=c[val][i];
      i++;

      cx=(cx*width)+x;
      if (cx<1)   cx=1;
      if (cx>254) cx=254;
      cy=(cy*height)+y;
      if (cy<1)   cy=1;
      if (cy>254) cy=254;

      printf("0x%2.2x,0x%2.2x",cx,cy);
	  NumCount += 2;
#if COUNT
	if (fEnabled)
		RecordXYPair(cx, cy);
#endif
	  n=n+2;
    }
      printf(",");
  }
  printf("0x%2.2x",term);
  NumCount++;
  if (force || ((NumCount % 2) == 0)) {
	  if (force && ((NumCount %2) == 1)) {
		  printf(",0x00");
	  }
	  printf("\n");
	  NumCount = 0;
	} else {
	printf(",");
  }
}

void MakeChar(char *name, int x, int y, int height, int width, int val, int term, int force, int fEnabled)
{
	int i;
	int n;
	int	c0[]={255};
	int	c1[]={-1,-3,-1,-1,1,-1,1,-3,-1,-3,255};									//dot
	int c2[]={-2,-3,2,3,255};													//slash
	int c3[]={-2,1,-2,2,-1,2,-1,1,-2,1,-255,-2,-1,-2,-2,-1,-2,-1,-1,-2,-1,255};	//colon
	int c4[]={-2,0,1,0,255};													//dash
	int c5[]={1,1,255};															//space
	int c6[]={1,1,255};															//blank
	int *c[]={c0,c1,c2,c3,c4,c5,c6};
	int cx;
	int cy;

  if (val>6) return;

  if (name != NULL) {
    printf("%-15s .db ",name);
  } else {
		if ((NumCount % 2) == 0) {
			printf("                .db ");
		}
  }

  i=0;
  n=0;
  while ((cx = c[val][i]) != 255) {
    i++;
    if (cx == -255) {
      printf("0x00");
	  NumCount++;
      n=n+1;
    } else {
      cy=c[val][i];
      i++;

      cx=(cx*width)+x;
      if (cx<1)   cx=1;
      if (cx>254) cx=254;
      cy=(cy*height)+y;
      if (cy<1)   cy=1;
      if (cy>254) cy=254;

      printf("0x%2.2x,0x%2.2x",cx,cy);
	  NumCount += 2;
#if COUNT
	if (fEnabled)
		RecordXYPair(cx, cy);
#endif
	  n=n+2;
    }
      printf(",");
  }
  printf("0x%2.2x",term);
  NumCount++;
  if (force || ((NumCount % 2) == 0)) {
	  if (force && ((NumCount %2) == 1)) {
		  printf(",0x00");
	  }
	  printf("\n");
	  NumCount = 0;
	} else {
	printf(",");
  }
}

void MakeRomanLetter(char *name, int x, int y, int val, int term, int force, int fEnabled)
{
	int i;
	int n;
	int c0[]={255};
	int	c1[]={-4,-10,4,-10,-255, 0,-10, 0,10, -255, -4,10, 4,10,255};	// I
	int	c2[]={-6,-10,6,-10,-255, -2,-10, -2,10, -255, 2,-10, 2,10, -255, -6,10, 6,10, 255};	// II
	int	c3[]={-9,-10,9,-10,-255, -5,-10, -5,10, -255, 0,-10,0,10,-255, 5,-10, 5,10, -255, -9,10, 9,10, 255};	// III
	int	c4[]={-10,-10,10,-10,-255, -6,-10,-6,10,-255, -1,10, 3,-10,7,10,-255, -10,10,10,10, 255};	// IV
	int	c5[]={-8,-10,8,-10,-255, -4,10,0,-10,4,10,-255, -8,10,8,10, 255};	// V
	int	c6[]={-10,-10,10,-10,-255, -6,10,-2,-10,2,10,-255, 6,-10,6,10,-255, -10,10,10,10, 255};	// VI
	int	c7[]={-11,-10,11,-10,-255, -8,10, -4,-10,0,10,-255, 3,-10,3,10,-255, 7,-10,7,10,-255, -11,10,11,10, 255};	// VII
	int	c8[]={-14,-10,14,-10,-255, -9,10,-5,-10,-1,10,-255, 3,-10,3,10,-255, 7,-10,7,10,-255,
														11,-10,11,10,-255, -14,10,14,10, 255};	// VIII
	int	c9[]={-9,-10,9,-10,-255, -6,-10,-6,10,-255, -2,-10,6,10,-255, -2,10,6,-10,-255, -9,10,9,10,255};		// IX
	int	c10[]={-7,-10,7,-10,-255, -4,-10,4,10,-255, -4,10,4,-10,-255, -7,10,7,10,255};		// X
	int	c11[]={-9,-10,9,-10,-255, -6,-10,2,10,-255, -6,10,2,-10,-255, 5,-10,5,10,-255, -9,10,9,10,255};		// XI
	int	c12[]={-10,-10,10,-10,-255, -7,-10,0,10,-255, -7,10,0,-10,-255, 3,-10,3,10,-255, 7,10,7,-10,-255, -10,10,10,10,255};		// XII
//			     I  II III IV V  VI VII VIII IX X   XI  XII
	int *c[]={c0,c1,c2,c3, c4,c5,c6,c7, c8,  c9,c10,c11,c12};
	int cx;
	int cy;

  if (val>12) return;

  if (name != NULL) {
    printf("%-15s .db ",name);
  } else {
		if ((NumCount % 2) == 0) {
			printf("                .db ");
		}
  }

  i=0;
  n=0;
  while ((cx = c[val][i]) != 255) {
    i++;
    if (cx == -255) {
      printf("0x00");
	  NumCount++;
      n=n+1;
    } else {
      cy=c[val][i];
      i++;

      cx=(cx)+x;
      if (cx<1)   cx=1;
      if (cx>254) cx=254;
      cy=(cy)+y;
      if (cy<1)   cy=1;
      if (cy>254) cy=254;

      printf("0x%2.2x,0x%2.2x",cx,cy);
	  NumCount += 2;
#if COUNT
	if (fEnabled)
		RecordXYPair(cx, cy);
#endif
	  n=n+2;
    }
      printf(",");
  }
  printf("0x%2.2x",term);
  NumCount++;
  if (force || ((NumCount % 2) == 0)) {
	  if (force && ((NumCount %2) == 1)) {
		  printf(",0x00");
	  }
	  printf("\n");
	  NumCount = 0;
	} else {
	printf(",");
  }
}

void MakeCalibrationData(char *pName)
{
	printf("%-15s .db ",pName);
	printf("1,0xfe,1,1,0xfe,1,0xfe,0xfe,1,0xfe,0xfe,1,1,1,0xfe,0xfe,1,0xfe,1,1,0xfe,1,0xfe,0xfe, 0xff,0\n");
}

